# rambshell.tcl  -*- TCL -*-
# 	$Id: rambshell.tcl,v 1.46 2002/06/13 18:21:21 ramsan Exp $	
#---------------------------------------------------------------------------
#This file is written in TCL lenguage 
#For more information about TCL look at: http://www.scriptics.com
#
#At least two procs must be in this file:
#
#    InitGIDProject dir - Will be called whenever a project is begun to be used.
#           where dir is the project's directory
#    EndGIDProject - Will be called whenever a project ends to be used.
#
#For more information about GID internals, check the program scripts.
#---------------------------------------------------------------------------


proc MyBitmaps { dir { type "DEFAULT INSIDELEFT"} } {
    global MyBitmapsNames MyBitmapsCommands MyBitmapsHelp ProblemTypePriv ProgramName

    set MyBitmapsNames(0) "images/constraints.gif images/connections.gif images/l1.gif \
	    images/loads.gif images/section.gif images/section-steel.gif images/concrete.gif \
	    images/prdata.gif images/drawconstraints.gif images/drawloads.gif images/mesh.gif \
	    images/calc.gif images/info.gif"
    set MyBitmapsCommands(0) [list [list -np- GidOpenConditions Constraints] \
	    [list -np- GidOpenConditions Connections] \
	    {} [list -np- GidOpenConditions Loads] \
	    [list -np- GidOpenConditions "General properties"] \
	    [list -np- GidOpenConditions "Steel properties"] \
	    [list -np- GidOpenConditions "Concrete properties"] \
	    [list -np- GidOpenProblemData "Problem data"] \
	    [list -np- LoadCasesDraw constraints] \
	    [list -np- LoadCasesDraw loads] \
	    "Meshing generate" \
	    "Utilities Calculate" "-np- PWViewOutput" ]
    set MyBitmapsHelp(0) { "Assign constraints" "Assign inter-element connections" {} "Assign loads" \
	    "Assign section/material Properties" \
	    "Assign section/material Properties for steel sections" \
	    "Assign section/material Properties for concrete sections" \
	    "Problem data" "Draw constraints" "Draw loads" "Generate mesh" "Calculate" \
	    "View process info"}
    
    set MyBitmapsNames(0,2) ""
    set MyBitmapsCommands(0,2) ""

    # prefix values:
    #          Pre        Only active in the preprocessor
    #          Post       Only active in the postprocessor
    #          PrePost    Active Always

    set prefix Pre

    set ProblemTypePriv(toolbarwin) [CreateOtherBitmaps MyBar "My toolbar" \
	    MyBitmapsNames MyBitmapsCommands \
	    MyBitmapsHelp $dir "MyBitmaps [list $dir]" $type $prefix]
    AddNewToolbar "$ProgramName bar" ${prefix}MyBarWindowGeom \
	    "MyBitmaps [list $dir]"

    # button number 3
    set ProblemTypePriv(loadcasewin) $ProblemTypePriv(toolbarwin).2

    if { [info exists ProblemTypePriv(loadcaseimg1)] } {
	ChangeToLoadCase 1
    }

}

proc EndMyBitmaps {} {
    global ProblemTypePriv ProgramName
    
    ReleaseToolbar "$ProgramName bar"
    rename MyBitmaps ""
    
    catch { destroy $ProblemTypePriv(toolbarwin) }
}

proc InitGIDProject { dir } {
    global MenuNames MenuEntries MenuCommands MenuAcceler
    global MenuNamesP MenuEntriesP MenuCommandsP MenuAccelerP
    global ProgramName VersionNumber ProblemTypePriv

    set ProgramName Rambshell
    set VersionNumber 4.0

    catch { unset ProblemTypePriv }

    set ProblemTypePriv(problemtypedir) $dir

    set num [lsearch $MenuNamesP "View results"]
    if { $num == -1 } {
	WarnWin "$ProgramName is not compatible with this GiD version"
	return
    }

    catch {
	#Splash $dir
    }

    foreach i [array names MenuEntriesP $num,*] { unset MenuEntriesP($i) }
    foreach i [array names MenuCommandsP $num,*] { unset MenuCommandsP($i) }
    set MenuEntriesP($num) ""
    set MenuCommandsP($num) "-np- CreatePostProcessMenu %W"

    set ipos [lsearch $MenuNames Help]
    if { $ipos != -1 } {
	set MenuEntries($ipos) [linsert $MenuEntries($ipos) 0 "RAM Series tutorials" \
		"RAM Series help" --- ]
	set MenuCommands($ipos) [linsert $MenuCommands($ipos) 0 \
		[list -np- HelpWindow "CUSTOM_HELP_FILE" \
		[file join $dir ../info ramseries_tutorials.html]] \
		[list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
		[file join $dir ../info ramseries_toc.html] \
		     [file join $dir ../info title.html]] ""]
	set MenuAcceler($ipos) [linsert $MenuAcceler($ipos) 0 "" "" ""]

	lappend MenuEntries($ipos) "About RAM Series"
	lappend MenuCommands($ipos) [list -np- Splash $dir 0]
	lappend MenuAcceler($ipos) ""
    }

    set ipos [lsearch $MenuNamesP Help]
    if { $ipos != -1 } {
	set MenuEntriesP($ipos) [linsert $MenuEntriesP($ipos) 0 "RAM Series tutorials" \
		"RAM Series help" --- ]
	set MenuCommandsP($ipos) [linsert $MenuCommandsP($ipos) 0 \
				      [list -np- HelpWindow "CUSTOM_HELP_FILE" \
					   [file join $dir ../info ramseries_tutorials.html]] \
				      [list -np- HelpWindow "CUSTOM_HELP_REFERENCE" \
					   [file join $dir ../info ramseries_toc.html] \
					   [file join $dir ../info title.html]] ""]
	set MenuAccelerP($ipos) [linsert $MenuAccelerP($ipos) 0 "" "" ""]

	lappend MenuEntriesP($ipos) "About RAM Series"
	lappend MenuCommandsP($ipos) [list -np- Splash $dir 0]
	lappend MenuAccelerP($ipos) ""
    }

    MyBitmaps $dir

    GidChangeDataLabel "Data units" ""
    GidChangeDataLabel "Interval" ""
    GidChangeDataLabel "Conditions" ""
    GidChangeDataLabel "Materials" ""
    GidChangeDataLabel "Interval Data" ""
    GidChangeDataLabel "Problem Data" ""
    
    GidAddUserDataOptions Constraints "GidOpenConditions Constraints" 2
    GidAddUserDataOptions Connections "GidOpenConditions Connections" 3
    GidAddUserDataOptions "---" "" 4
    GidAddUserDataOptions "Load cases" "LoadCaseWindow" 5
    GidAddUserDataOptions Loads "GidOpenConditions Loads" 6
    GidAddUserDataOptions "---" "" 7

    GidAddUserDataOptions "General properties" "GidOpenConditions {General properties}" 8
    GidAddUserDataOptions "Steel properties" "GidOpenConditions {Steel properties}" 9
    GidAddUserDataOptions "Concrete properties" "GidOpenConditions {Concrete properties}" 10
    
    GidAddUserDataOptions "---" "" 11
    GidAddUserDataOptions "Problem data" "GidOpenProblemData {Problem data}" 12
    GidAddUserDataOptions "Tdyn Pressures" [list Pmesh::Main $dir] 13
    GidAddUserDataOptions "---" "" 14
#      GidAddUserDataOptions "Layers to Post sets" "AssignMaterialsFromLayers" 10
#      GidAddUserDataOptions "---" "" 11
   
    catch {
	GidShowBook materials "" 0
	GidShowBook intvdata "" 0
    }

    source [file join $dir scripts [string tolower $ProgramName]-2.tcl]

    source [file join $dir scripts Concrete.tcl]
    uplevel #0 [list source [file join $dir scripts rtflib.tcl]]
    source [file join $dir scripts HelpViewer.tcl]
    source [file join $dir scripts pmesh.tcl]
    source [file join $dir scripts SteelSections.tcl]

    if { [catch {
	LoadCasesInit $dir
    }] } {
	WarnWin "$ProgramName is not compatible with this GiD version -a-"
	return
    }
    set num [lsearch $MenuNamesP Help]
    if { $num == -1 } {
	WarnWin "$ProgramName is not compatible with this GiD version -a-"
	return
    }
    set num_m1 [expr $num+1]
    foreach i [array names MenuEntriesP $num*] {
	set rest ""
	regexp {^[0-9]*(.*)$} $i {} rest
	set MenuEntriesP($num_m1$rest) $MenuEntriesP($i)
	unset MenuEntriesP($i)
	set MenuCommandsP($num_m1$rest) $MenuCommandsP($i)
	unset MenuCommandsP($i)
	set MenuAccelerP($num_m1$rest) $MenuAccelerP($i)
	unset MenuAccelerP($i)
    }
    set MenuNamesP [linsert $MenuNamesP $num "Concrete/Steel"]
    set MenuEntriesP($num) [list "Dimension concrete" "Steel report"]
    set MenuCommandsP($num) [list [list -np- DimensionConcrete $dir] [list -np- ViewSteelReport $dir]]
    set MenuAccelerP($num) [list ""]

    set ProblemTypePriv(PropertyToPost) [.central.s info variables PropertyToPost]
    if { $ProblemTypePriv(PropertyToPost) != "Layer" } {
	.central.s process escape escape escape escape Utilities Variables PropertyToPost \
		Layer escape escape
    }

    global GidPriv
    set GidPriv(ProgName) $ProgramName
    ChangeWindowTitle
    CreateTopMenus
}

proc EndGIDProject {} {
    global ProblemTypePriv
    EndMyBitmaps

#      if { [info exists ProblemTypePriv(PropertyToPost)] && \
#  	    $ProblemTypePriv(PropertyToPost) != "Layer" } {
#  	.central.s process escape escape escape escape Utilities Variables PropertyToPost \
#  		$ProblemTypePriv(PropertyToPost) escape escape
#      }

    if { [info exists ProblemTypePriv(condfuns)] } {
	foreach i $ProblemTypePriv(condfuns) {
	    rename $i {}
	    rename $i-base $i
	}
    }
    catch { destroy .gid.loadcases }

    global GidPriv
    set GidPriv(ProgName) GiD
    ChangeWindowTitle ""
}

proc InitGIDPostProcess {} {
    global BarresPriv
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]
    foreach i [array names MenuEntriesP $num,*] { unset MenuEntriesP($i) }
    foreach i [array names MenuCommandsP $num,*] { unset MenuCommandsP($i) }
    set MenuEntriesP($num) ""
    set MenuCommandsP($num) "-np- CreatePostProcessMenu %W"
    catch { unset BarresPriv(problemtype) }
    set BarresPriv(EnteringPost) 1
}


proc HelpOnRamSeries { dir } {

    WarnWin [join [list "To obtain help for RAM Series, check in directory: " \
	    "'[file dirname $dir]' for a file called RamSeries.pdf"] ""]

}

proc Splash { dir { autodestroy 1 } } {
    global GIDDEFAULT VersionNumber ProgramName

    if { [.gid.central.s disable windows] } { return }

    if { [ winfo exist .splash]} {
	destroy .splash
	update
    }

    toplevel .splash
    wm withdraw .splash
        
    set im [image create photo -file [ file join $dir images \
	    [string tolower $ProgramName]-open.gif]]
    set x [expr [winfo screenwidth .splash]/2-[image width $im]/2]
    set y [expr [winfo screenheight .splash]/2-[image height $im]/2]

    wm geom .splash +$x+$y
    wm transient .splash .gid
    wm overrideredirect .splash 1
    pack [label .splash.l -image $im -relief ridge -bd 2]
    
    label .splash.lv -text $VersionNumber -font "times 32"
    place .splash.lv -x 70 -y 150
    bind .splash <1> "destroy .splash"
    bind .splash <KeyPress> "destroy .splash"
    grab .splash
    focus .splash
    update
    wm deiconify .splash
    raise .splash .gid
    update
    if { $autodestroy } {
	after 2000 "if { [ winfo exist .splash] } { 
	    destroy .splash
	}"
    }
}

proc ViewSteelReport { dir } {

    set projdir [ConcretePrefs::GiveProjectDir]
    if { $projdir != "" } {
	set basename [file root [file tail $projdir]]
	set filename [file join $projdir ${basename}_steel.html]
    } else {
	WarnWin "There is no report to visualize"
	return
    }

    if { ![file exists $filename] } {
	WarnWin "There is no report to visualize"
	return
    }
    set top .gid
    set x [expr [winfo x $top]+[winfo width $top]/2-350]
    set y [expr [winfo y $top]+[winfo height $top]/2-300]
    set geom 650x600+${x}+$y
    
    HelpWindowRam -geomhelp $geom -tt -title "Steel report" -win .gid.steelreport $filename
    
}

proc SetVarsForBeams {} {
    global MenuEntriesP MenuCommandsP MenuNamesP

    set curr [.central.s info postprocess get cur_analisis]
    set restypes [.central.s info postprocess get results_list Contour_Fill $curr 1]

    set num [lsearch $MenuNamesP "View results"]

    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }
    set MenuEntriesP($num) ""
    set MenuCommandsP($num) ""

    if { [lsearch $restypes Z'_Momentus*] != -1 } {
	lappend MenuEntriesP($num) {Beam Diagrams} Deformed Reactions {Local axes}
	lappend MenuCommandsP($num) {} {-np- DisplayDeformed} \
	    {} {-np- DisplayVectors Local_Axes_Beam All}
    }
    lappend MenuEntriesP($num) --- {No results}
    lappend MenuCommandsP($num) {} {-np- StopResults} 

    if { [lsearch $restypes Resistance_factor] != -1 } {
	set MenuEntriesP($num) [linsert $MenuEntriesP($num) end-2 "Steel Resistance factor"]
	set MenuCommandsP($num) [linsert $MenuCommandsP($num) end-2 ""]
	set idx [expr [llength $MenuCommandsP($num)]-3]
	set MenuEntriesP($num,$idx) [list Regions Values]
	set MenuCommandsP($num,$idx) [list "-np- DisplayContourRanges Resistance_factor \
                   Resistance_factor" "-np- DisplayContourFill Resistance_factor \
                   Resistance_factor"]
    }
    
    set res "Axial_Force Y'_Shear Z'_Shear Torsor Y'_Momentus Z'_Momentus"
    set resp "B_Axial_Force Y'_Shear Z'_Shear Torsor Y'_Momentus Z'_Momentus"

    if { [lsearch $restypes Z'_Momentus_min*] != -1 } {
	set MenuEntriesP($num,0) "$res"
	set MenuCommandsP($num,0) ""
	set ic 0
	foreach i $resp {
	    set MenuEntriesP($num,0,$ic) "Min Max --- Both"
	    set MenuCommandsP($num,0,$ic) [list "-np- DisplayVectorLineDiagram ${i}_min" \
					       "-np- DisplayVectorLineDiagram ${i}_max" \
					       {} "-np- DisplayVectorLineDiagram ${i}"]
	    incr ic
	}
    } elseif { [lsearch $restypes Z'_Momentus*] != -1 } {
	set MenuEntriesP($num,0) "$res"
	set MenuCommandsP($num,0) ""
	foreach i $resp {
	    lappend MenuCommandsP($num,0) "-np- DisplayVectorLineDiagram $i"
	}
    }
    set ipos [lsearch $MenuEntriesP($num) Reactions]
    if { $ipos != -1 } {
	set MenuEntriesP($num,$ipos) { Force Momentus Compose --- "Force x" "Force y" "Force z" }
	set MenuCommandsP($num,$ipos) { \
					    {-np- DisplayVectors Reactions |Reactions|} \
					    {-np- DisplayVectors M_Reactions "|M_Reactions|"} \
					    {-np- AddReactionsWindow}  \
					    {} {-np- DisplayVectors Reactions Px} \
					    {-np- DisplayVectors Reactions Py} \
					    {-np- DisplayVectors Reactions Pz} } 
    }
}

proc AddVarsForBeams {} {
    global MenuNamesP MenuEntriesP MenuCommandsP
    
    set curr [.central.s info postprocess get cur_analisis]
    set restypes [.central.s info postprocess get results_list Contour_Fill $curr 1]

     set num [lsearch $MenuNamesP "View results"]

    if { [lsearch $restypes Z'_Momentus*] != -1 } {
	lappend MenuEntriesP($num) --- {Beam Diagrams} {Beam deformed} {Beam Local axes}
	lappend MenuCommandsP($num) {} {} \
	    {-np- DisplayVectorLineDiagram BDisplacements} \
	    {-np- DisplayVectors Local_Axes_Beam All}
    }
    if { [lsearch $restypes Resistance_factor] != -1 } {
	lappend MenuEntriesP($num) "Resistance factor"
	lappend MenuCommandsP($num) "-np- DisplayContourFill \
                   Resistance_factor Resistance_factor"
    }
    
    set res "Axial_Force Y'_Shear Z'_Shear Torsor Y'_Momentus Z'_Momentus"
    set resp "B_Axial_Force Y'_Shear Z'_Shear Torsor Y'_Momentus Z'_Momentus"
    
    set ipos [lsearch $MenuEntriesP($num) {Beam Diagrams}]

    if { $ipos != -1 } {
	if { [lsearch $restypes Z'_Momentus_min*] == -1 } {
	    set MenuEntriesP($num,$ipos) "$res"
	    set MenuCommandsP($num,$ipos) ""
	    foreach i $resp {
		lappend MenuCommandsP($num,$ipos) "-np- DisplayVectorLineDiagram $i"
	    }
	} else {
	    set MenuEntriesP($num,$ipos) "$res"
	    set MenuCommandsP($num,$ipos) ""
	    set ic 0
	    foreach i $resp {
		set MenuEntriesP($num,$ipos,$ic) "Min Max --- Both"
		set MenuCommandsP($num,$ipos,$ic) [list "-np- DisplayVectorLineDiagram ${i}_min" \
						       "-np- DisplayVectorLineDiagram ${i}_max" \
						       {} "-np- DisplayVectorLineDiagram ${i}"]
		incr ic
	    }
	}
    }
}

proc SetVarsForShellsXY {} {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]
    
    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }
 
    set MenuEntriesP($num) "Displacements {Axial Force} Momentus Shear --- \
	    Deformed Reactions {Local axes} {No results} --- {Integrate/graphs}"
    set MenuCommandsP($num) { {} {} {} {} {} {-np- DisplayDeformed} \
	    {} {-np- DisplayVectors Local_Axes All} \
	    {-np- StopResults} {} {-np- IntegrateWindow} }
   
    set MenuEntriesP($num,0) { X Y Z |Displacements|}
    set MenuEntriesP($num,1) { Nx' Ny' Nxy' }
    set MenuEntriesP($num,2) { Mx' My' Mxy' }
    set MenuEntriesP($num,3) { Qx' Qy' }

    for {set i 0} {$i<4} {incr i} {
	set MenuCommandsP($num,$i) ""
	regsub -all { } [lindex $MenuEntriesP($num) $i] {_} res
	foreach j $MenuEntriesP($num,$i) {
	    lappend MenuCommandsP($num,$i) "-np- DisplayContourFill $res $j"
	}
    }
    set MenuEntriesP($num,6) { Force Momentus Compose --- "Force x" "Force y" "Force z" }
    set MenuCommandsP($num,6) { \
	    {-np- DisplayVectors Reactions |Reactions|} \
	    {-np- DisplayVectors M_Reactions "|M_Reactions|"} \
	    {-np- AddReactionsWindow}  \
	    {} {-np- DisplayVectors Reactions Px} \
	    {-np- DisplayVectors Reactions Py} \
	    {-np- DisplayVectors Reactions Pz} } 
}


proc SetVarsForSolids {} {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]

    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }

    set MenuEntriesP($num) "Displacements Strengths {Von Misses} --- \
	    Deformed Reactions --- {No results}"
    set MenuCommandsP($num) { {} {} {-np- DisplayContourFill Von_Misses Von_Misses} {} \
	    {-np- DisplayDeformed}  \
	    {} {} {-np- StopResults} }
   
    set MenuEntriesP($num,0) { X Y Z |Displacements|}
    set MenuEntriesP($num,1) {Sx Sy Sz Txy Txz Tyz Si-Strengths Sii-Strengths Siii-Strengths}

    for {set i 0} {$i<2} {incr i} {
	set MenuCommandsP($num,$i) ""
	regsub -all { } [lindex $MenuEntriesP($num) $i] {_} res
	foreach j $MenuEntriesP($num,$i) {
	    lappend MenuCommandsP($num,$i) "-np- DisplayContourFill $res $j"
	}
    }
    set MenuEntriesP($num,5) { Force Compose --- "Force x" "Force y" "Force z" }
    set MenuCommandsP($num,5) { \
	    {-np- DisplayVectors Reactions |Reactions|} \
	    {-np- AddReactionsWindow} \
	    {} {-np- DisplayVectors Reactions Px} \
	    {-np- DisplayVectors Reactions Py} \
	    {-np- DisplayVectors Reactions Pz} }
}

proc SetVarsForShellsMain {} {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]

    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }

    set MenuEntriesP($num) "Displacements {Main Ax. Force} {Main Momentus} \
	    --- Deformed Reactions {Local axes} {No results} --- {Integrate/graphs}"
    set MenuCommandsP($num) { {} {} {} {} {-np- DisplayDeformed} \
	    {-np- DisplayVectors Reactions |Reactions|} \
	    {-np- DisplayVectors Local_Axes All } \
	    {-np- StopResults}  {} {-np- IntegrateWindow} }
   
    set MenuEntriesP($num,0) { X Y Z |Displacements|}
    foreach j $MenuEntriesP($num,0) {
	lappend MenuCommandsP($num,0) "-np- DisplayContourFill Displacements $j"
    }

    set MenuEntriesP($num,1) { Both --- N11 N22 --- "N11 vector" "N22 vector"}
    set MenuCommandsP($num,1) { \
	    {-np- DisplayVectors Mat_Ax._Force All} {} \
	    {-np- DisplayContourFill Main_Ax._Force N11} \
	    {-np- DisplayContourFill Main_Ax._Force N22} {} \
	    {-np- DisplayVectors Mat_Ax._Force Si} \
	    {-np- DisplayVectors Mat_Ax._Force Sii} }

    set MenuEntriesP($num,2) { M11 M22 --- "Local axes" --- Qm1 Qm2 }
    set MenuCommandsP($num,2) { \
	    {-np- DisplayContourFill Main_Momentus M11}  \
	    {-np- DisplayContourFill Main_Momentus M22} {} \
	    {-np- DisplayVectors L_Axes_Momentus All } {} \
	    {-np- DisplayContourFill Shear_in_Main Qm1}  \
	    {-np- DisplayContourFill Shear_in_Main Qm2}}
    set MenuEntriesP($num,5) { Force Momentus Compose --- "Force x" "Force y" "Force z" }
    set MenuCommandsP($num,5) { \
	    {-np- DisplayVectors Reactions |Reactions|} \
	    {-np- DisplayVectors M_Reactions "|M_Reactions|"} \
	    {-np- AddReactionsWindow}  \
	    {} {-np- DisplayVectors Reactions Px} \
	    {-np- DisplayVectors Reactions Py} \
	    {-np- DisplayVectors Reactions Pz} } 

}

proc SetVarsForShellsBoth {} {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]

    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }

    set MenuEntriesP($num) "Displacements {Axial Force} Momentus Shear --- \
	    {Main Ax. Force} {Main Momentus} \
	    --- Deformed Reactions {Local axes} {No results} --- {Integrate/graphs}"
    set MenuCommandsP($num) { {} {} {} {} {} {} {} {} {-np- DisplayDeformed} \
	    {} {-np- DisplayVectors Local_Axes All } \
	    {-np- StopResults}  {} {-np- IntegrateWindow} }
   
    set MenuEntriesP($num,0) { X Y Z |Displacements|}
    set MenuEntriesP($num,1) { Nx' Ny' Nxy' }
    set MenuEntriesP($num,2) { Mx' My' Mxy' }
    set MenuEntriesP($num,3) { Qx' Qy' }

    for {set i 0} {$i<4} {incr i} {
	set MenuCommandsP($num,$i) ""
	regsub -all { } [lindex $MenuEntriesP($num) $i] {_} res
	foreach j $MenuEntriesP($num,$i) {
	    lappend MenuCommandsP($num,$i) "-np- DisplayContourFill $res $j"
	}
    }

    set MenuEntriesP($num,5) { Both --- N11 N22 --- "N11 vector" "N22 vector"}
    set MenuCommandsP($num,5) { \
	    {-np- DisplayVectors Mat_Ax._Force All} {} \
	    {-np- DisplayContourFill Main_Ax._Force N11} \
	    {-np- DisplayContourFill Main_Ax._Force N22} {} \
	    {-np- DisplayVectors Mat_Ax._Force Si} \
	    {-np- DisplayVectors Mat_Ax._Force Sii} }

    set MenuEntriesP($num,6) { M11 M22 --- "Local axes" --- Qm1 Qm2 }
    set MenuCommandsP($num,6) { \
	    {-np- DisplayContourFill Main_Momentus M11}  \
	    {-np- DisplayContourFill Main_Momentus M22} {} \
	    {-np- DisplayVectors L_Axes_Momentus All } {} \
	    {-np- DisplayContourFill Shear_in_Main Qm1}  \
	    {-np- DisplayContourFill Shear_in_Main Qm2}} 
    
    set MenuEntriesP($num,9) { Force Momentus Compose --- "Force x" "Force y" "Force z" }
    set MenuCommandsP($num,9) { \
	    {-np- DisplayVectors Reactions |Reactions|} \
	    {-np- DisplayVectors M_Reactions "|M_Reactions|"} \
	    {-np- AddReactionsWindow}  \
	    {} {-np- DisplayVectors Reactions Px} \
	    {-np- DisplayVectors Reactions Py} \
	    {-np- DisplayVectors Reactions Pz} } 
}

proc SetVarsForShells {} {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set curr [.central.s info postprocess get cur_analisis]
    set restypes [.central.s info postprocess get results_list Contour_Fill $curr 1]

    set num [lsearch $MenuNamesP "View results"]

    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }

    set MenuEntriesP($num) [list "Displacements"]
    set MenuCommandsP($num) [list ""]
    set MenuEntriesP($num,0) { X Y Z |Displacements|}
    foreach j $MenuEntriesP($num,0) {
	lappend MenuCommandsP($num,0) "-np- DisplayContourFill Displacements $j"
    }
    if { [lsearch $restypes Axial_Force*] != -1 } {
	lappend MenuEntriesP($num) "Axial Force" Momentus Shear ---
	lappend MenuCommandsP($num) "" "" "" ""
	set MenuEntriesP($num,1) { Nx' Ny' Nxy' }
	set MenuEntriesP($num,2) { Mx' My' Mxy' }
	set MenuEntriesP($num,3) { Qx' Qy' }

	for {set i 1} {$i<4} {incr i} {
	    set MenuCommandsP($num,$i) ""
	    regsub -all { } [lindex $MenuEntriesP($num) $i] {_} res
	    foreach j $MenuEntriesP($num,$i) {
		lappend MenuCommandsP($num,$i) "-np- DisplayContourFill $res $j"
	    }
	}
	set ip 5
    } else { set ip 1 }

    if { [lsearch $restypes Main_Ax._Force*] != -1 } {
	lappend MenuEntriesP($num) "Main Ax. Force" "Main Momentus" ---
	lappend MenuCommandsP($num) "" "" ""

	set MenuEntriesP($num,$ip) { Both --- N11 N22 --- "N11 vector" "N22 vector"}
	set MenuCommandsP($num,$ip) { \
		{-np- DisplayVectors Mat_Ax._Force All} {} \
		{-np- DisplayContourFill Main_Ax._Force N11} \
		{-np- DisplayContourFill Main_Ax._Force N22} {} \
		{-np- DisplayVectors Mat_Ax._Force Si} \
		{-np- DisplayVectors Mat_Ax._Force Sii} }
	incr ip
	set MenuEntriesP($num,$ip) { M11 M22 --- "Local axes" --- Qm1 Qm2 }
	set MenuCommandsP($num,$ip) { \
		{-np- DisplayContourFill Main_Momentus M11}  \
		{-np- DisplayContourFill Main_Momentus M22} {} \
		{-np- DisplayVectors L_Axes_Momentus All } {} \
		{-np- DisplayContourFill Shear_in_Main Qm1}  \
		{-np- DisplayContourFill Shear_in_Main Qm2}} 
	incr ip 2
    }

    if { [lsearch $restypes Strengths_Up*] != -1 } {
	lappend MenuEntriesP($num) "Strengths Up" "Strengths Down" "Von Misses Up" \
		"Von Misses Down" ---
	lappend MenuCommandsP($num)  "" "" \
		{-np- DisplayContourFill Von_Misses_Up Von_Misses_Up} \
		{-np- DisplayContourFill Von_Misses_Down Von_Misses_Down} ""
	set MenuEntriesP($num,$ip) {Sx Sy Sz Txy Txz Tyz Si-Strengths Sii-Strengths Siii-Strengths}
	foreach j $MenuEntriesP($num,$ip) {
	    lappend MenuCommandsP($num,$ip) "-np- DisplayContourFill Strengths_Up $j"
	}
	incr ip
	set MenuEntriesP($num,$ip) {Sx Sy Sz Txy Txz Tyz Si-Strengths Sii-Strengths Siii-Strengths}
	foreach j $MenuEntriesP($num,$ip) {
	    lappend MenuCommandsP($num,$ip) "-np- DisplayContourFill Strengths_Down $j"
	}
	incr ip 4
    }

    lappend MenuEntriesP($num) Deformed Reactions {Local axes} {No results} --- {Integrate/graphs}
    lappend MenuCommandsP($num) {-np- DisplayDeformed} \
	    {} {-np- DisplayVectors Local_Axes All } \
	    {-np- StopResults}  {} {-np- IntegrateWindow}

    incr ip
    set MenuEntriesP($num,$ip) { Force Momentus Compose --- "Force x" "Force y" "Force z" }
    set MenuCommandsP($num,$ip) { \
	    {-np- DisplayVectors Reactions |Reactions|} \
	    {-np- DisplayVectors M_Reactions "|M_Reactions|"} \
	    {-np- AddReactionsWindow}  \
	    {} {-np- DisplayVectors Reactions Px} \
	    {-np- DisplayVectors Reactions Py} \
	    {-np- DisplayVectors Reactions Pz} } 

}

proc SetVarsForNoneWin {} {
    WarnWin "There are no Results"
}

proc SetVarsForNone {} {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]

    foreach i [array names MenuEntriesP $num,*] {
	unset MenuEntriesP($i)
    }
    foreach i [array names MenuCommandsP $num,*] {
	unset MenuCommandsP($i)
    }

    set MenuEntriesP($num) "{No Results}"
    set MenuCommandsP($num) { {-np- SetVarsForNoneWin} }

}

proc AddloadcasesToPostMenu { w alla } {
    global stupidval
    set stupidval 1
    set curr [.central.s info postprocess get cur_analisis]

    $w del 0 end
    set inum 1
    foreach i $alla {
	if { $i == $curr } {
	    $w add checkbutton -label $i -variable stupidval -command "ChangeToAnalysis $i" \
		    -acc "Ctrl-$inum"
	} else {
	    $w add command -label $i -command "ChangeToAnalysis $i" \
		    -acc "Ctrl-$inum"
	}
	bind all <Control-KeyPress-$inum> "$w invoke $i"
	incr inum
    }
}

proc AddLoadCasesToMenu { alla } {
    global MenuNamesP MenuEntriesP MenuCommandsP

    set num [lsearch $MenuNamesP "View results"]

    set MenuEntriesP($num) [linsert $MenuEntriesP($num) 0 Loadcase ---]
    set MenuCommandsP($num) [linsert $MenuCommandsP($num) 0 \
	    [list -np- AddloadcasesToPostMenu %W $alla] {}]

    set names [array names MenuEntriesP $num,*]
    foreach i [lsort -decreasing $names] {
	regexp {^([0-9]+),([0-9]+)(.*)$} $i {} n1 n2 r
	incr n2 2
	set ni $n1,$n2$r
	set MenuEntriesP($ni) $MenuEntriesP($i)
	unset MenuEntriesP($i)
	set MenuCommandsP($ni) $MenuCommandsP($i)
	unset MenuCommandsP($i)
    }
    set MenuEntriesP($num,0) ""
    set MenuCommandsP($num,0) ""
    foreach i $alla {
	lappend MenuEntriesP($num,0) $i
	lappend MenuCommandsP($num,0) ""
    }
}

proc CreatePostProcessMenu { w } {
    global BarresPriv MenuNamesP

    set num [lsearch $MenuNamesP "View results"]

    set infoProj [lindex [.gid.central.s info Project] 4]
    if { $infoProj != "POSTUSE" && $infoProj != "GRAFUSE" } { return }


    set alla [.central.s info postprocess get all_analisis]
    set curr [.central.s info postprocess get cur_analisis]

    if { $BarresPriv(EnteringPost) == 1 } {
	set BarresPriv(EnteringPost) 0
    if { $curr == "Max_Min" } {
	set newa [lindex $alla end-1]
	if { $newa != "" } {
	    .central.s process escape escape escape escape Results AnalysisSel $newa \
		escape escape
	    set curr [.central.s info postprocess get cur_analisis]
	}
    }
    }

    if { [llength $alla] > 0 } {
	set restype [.central.s info postprocess get results_list Contour_Fill \
		$curr 1]
    } else { set restype "" }
    
    if { [lsearch $restype Strengths*] != -1 && [lsearch $restype Local_Axes*] == -1 } {
	set type solid
    } elseif { [lsearch $restype Momentus*] != -1 && \
	    [lsearch $restype Main_Ax._Force*] != -1 } {
	set type both
    } elseif { [lsearch $restype Momentus*] != -1 } {
	set type xy
    } elseif { [lsearch $restype Main_Ax._Force*] != -1 } {
	set type main
    } elseif { [lsearch $restype Y'_Momentus*] != -1 || \
		   [lsearch $restype Resistance_factor*] != -1 } {
	set type beams
    } else { set type none }

    if { $type != "beams" && [lsearch $restype Y'_Momentus*] != -1 } {
	set type2 beams
    } else { set type2  "" }

    if { ![info exists BarresPriv(problemtype)] || \
	    $BarresPriv(problemtype) != $type || \
	    $BarresPriv(problemtype2) != $type2 || [$w index end] == "none" } {
	switch $type {
	    solid SetVarsForSolids
	    beams SetVarsForBeams
	    both - xy - main SetVarsForShells
	    none SetVarsForNone
	}
	if { $type2 == "beams" } {
	    AddVarsForBeams
	}

	if { [llength $alla] > 1 && $type != "none"} {
	    AddLoadCasesToMenu $alla
	}

	$w del 0 end
	CreateCascadedMenu $w MenuEntriesP MenuCommandsP MenuAcceler $num
    }
    set BarresPriv(problemtype) $type
    set BarresPriv(problemtype2) $type2
}


proc StopResults {} {
    .central.s process escape escape escape escape Results Geometry \
	    NoResults escape
    .central.s process escape escape escape escape Results Geometry HideTheShow \
	    escape
}

# proc DisplayScalarLineDiagram { args } {
    
#     .central.s process escape escape escape escape Results LineDiagram \
# 	    ScalarDiagram $args

# }

proc RamSeriesSetCommentsForLineDiagram { result } {

    set aa [.central.s info post get comments]
    if { [lindex $aa 1] != "yes" } { return -1 }

    set DisableWarnLine [.central.s disable warnline]
    .central.s disable warnline 1

    

    .central.s process escape escape escape escape utilities comments Line1 %an
    set t "\"Line diagram of $result, factor %vf\""
    .central.s process Line2 $t
    .central.s process escape escape utilities comments automatic no
    .central.s process escape escape utilities comments comments yes

    if { !$DisableWarnLine } { .central.s disable warnline 0 }
    return 0
}

proc RamSeriesSetCommentsToAutomatic { result } {

    set DisableWarnLine [.central.s disable warnline]
    .central.s disable warnline 1

    .central.s process escape escape escape escape utilities comments automatic yes escape

    .central.s process escape escape escape escape Results ContOptions ChangeTitleLeg "\"$result\""

    if { !$DisableWarnLine } { .central.s disable warnline 0 }
}

proc DisplayVectorLineDiagram { result args } {
#      .central.s process escape escape escape escape Results LineDiagram \
#  	    Options ShowElevations NodesOnly LineDiagram VectorDiagram $args

    set anal [.central.s info postprocess get cur_analisis]

    set reslist [.central.s info postprocess get results_list Vector_Line_Diagram \
		 $anal 1]

    set ipos1 -1
    set ipos2 -1
    set i 0
    foreach res $reslist {
	if { [string match $result* $res] } {
	    if { $ipos1 == -1 } {
		set ipos1 $i
		set resultname $res
	    } else {
		set ipos2 $i
		set resultname $res
	    }
	}
	incr i
    }
    if { $ipos1 == -1 } {
	WarnWin "The results '$result' does not exist in loadcase '$anal'"
    } elseif { $ipos2 == -1 } {
	regsub -all _ $resultname { } resultname
	set DisableGraphics [.central.s disable graphics]
	.central.s disable graphics 1
	set fail [RamSeriesSetCommentsForLineDiagram $resultname]
	.central.s process escape escape escape escape Results Geometry NoResults Geometry Original
	.central.s process escape escape escape escape Results LineDiagram \
	    VectorDiagram $result $args
	if { !$fail } { RamSeriesSetCommentsToAutomatic $resultname }
	if { !$DisableGraphics } { .central.s disable graphics 0 }
	.central.s process redraw
    } else {
	regsub -all _ $resultname { } resultname
	regsub {min\s+} $resultname {} resultname
	set DisableGraphics [.central.s disable graphics]
	.central.s disable graphics 1
	set fail [RamSeriesSetCommentsForLineDiagram "$resultname Min Max"]
	set DisableWarnLine [.central.s disable warnline]
	.central.s disable warnline 1

	.central.s process escape escape escape escape utilities variables postresoverres yes
	.central.s process escape escape escape escape Results Geometry NoResults Geometry Original
	if { !$DisableWarnLine } { .central.s disable warnline 0 }
	.central.s process escape escape escape escape Results LineDiagram \
	    VectorDiagram [lindex $reslist $ipos1] $args
	.central.s process escape escape escape escape Results LineDiagram \
	    VectorDiagram [lindex $reslist $ipos2] $args
	set DisableWarnLine [.central.s disable warnline]
	.central.s disable warnline 1
	.central.s process escape escape escape escape utilities variables \
	    postresoverres no escape
	if { !$DisableWarnLine } { .central.s disable warnline 0 }
	if { !$fail } { RamSeriesSetCommentsToAutomatic "$resultname MM" }
	if { !$DisableGraphics } { .central.s disable graphics 0 }
	.central.s process redraw
    }
}

# proc DisplayVectorLineDiagram2 { result1 result2 args } {
# #      .central.s process escape escape escape escape Results LineDiagram \
# #  	    Options ShowElevations NodesOnly LineDiagram VectorDiagram $args

#     set anal [.central.s info postprocess get cur_analisis]

#     set reslist [.central.s info postprocess get results_list Vector_Line_Diagram \
# 		 $anal 1]

#     set ipos1 [lsearch $reslist $result1]
#     set ipos2 [lsearch $reslist $result2]

#     .central.s process escape escape escape escape utilities variables postresoverres yes
#     .central.s process escape escape escape escape Results Geometry NoResults Geometry Original
#     .central.s process escape escape escape escape Results LineDiagram \
# 	    VectorDiagram [lindex $reslist $ipos1] $args
#     .central.s process escape escape escape escape Results LineDiagram \
# 	    VectorDiagram [lindex $reslist $ipos2] $args
#     .central.s process escape escape escape escape utilities variables \
# 	    postresoverres no escape
# }

proc DisplayDeformedSD { args } {

    set DisableGraphics [.central.s disable graphics]
    .central.s disable graphics 1
    .central.s process escape escape escape escape Results Geometry NoResults Geometry Original
    set cur_anal [.gid.central.s info post get cur_analisis]
    .central.s process escape escape escape escape Results Geometry ShowDeformed \
	    $cur_anal 1 $args escape escape
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    .central.s process redraw
}

proc DisplayContourFill { args } {
    set DisableGraphics [.central.s disable graphics]
    .central.s disable graphics 1
    .central.s process escape escape escape escape Results Geometry NoResults Geometry Original
    .central.s process escape escape escape escape Results ContourFill $args \
	    escape
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    .central.s process redraw
}

proc DisplayContourRanges { args } {

    set DisableGraphics [.central.s disable graphics]
    .central.s disable graphics 1
    .central.s process escape escape escape escape Results Geometry NoResults Geometry Original
    .central.s process escape escape escape escape Results ContourRanges $args \
	escape
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    .central.s process redraw
}

proc DisplayVectors { args } {

    set DisableGraphics [.central.s disable graphics]
    .central.s disable graphics 1
    .central.s process escape escape escape escape Results Geometry NoResults Geometry Original
    .central.s process escape escape escape escape DisplayStyle Boundaries
    .central.s process escape escape escape escape Results DisplayVectors \
	    $args
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    .central.s process redraw
}

# proc DisplayGraphs { args } {

#     .central.s process escape escape escape escape Results Geometry NoResults Geometry Original
#     .central.s process escape escape escape escape Results Graphs BorderGraph \
# 	    LineVariation $args
# }

proc DisplayDeformed {} {

    set DisableGraphics [.central.s disable graphics]
    .central.s disable graphics 1
     set cur_anal [.gid.central.s info post get cur_analisis]
    set restype [.central.s info postprocess get results_list Contour_Fill \
		$cur_anal 1]

    if { [lsearch $restype Displacements*] != -1 } {
	DisplayDeformedSD Displacements
    } else {
	DisplayVectorLineDiagram BDisplacements
    }
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    .central.s process redraw
}
proc ChangeToAnalysis { newa } {
    
    set cur_anal [.gid.central.s info post get cur_analisis]
    set cur_step [.gid.central.s info post get cur_step $cur_anal] 
    set cur_res [.gid.central.s info post get cur_result]
    set cur_res_view  [.gid.central.s info post get cur_results_view]
    set cur_comp [.gid.central.s info post get cur_component]
    regsub -all {_} $cur_res_view {} pp
    set restcommand "$pp cur_res $cur_comp"
    switch $cur_res_view {
	"Iso_Surfaces" {
	    set values [.gid.central.s info post get iso_cur_result_values]
	    set restcommand "$pp exact cur_res $cur_comp [lindex $values 0] \
		    [lindex $values 1]"
	}
	"Stream_Lines" {
	    set restcommand ""
	}
	"No_Result" {
	    set restcommand ""
	}
	"Display_Vectors" {
	    set value [.gid.central.s info post get cur_vector_factor $cur_res_view \
		    $cur_res $cur_anal $cur_step ]
	    set restcommand "$restcommand $value escape"
	}
	"Vector_Line_Diagram" {
	    set value [.gid.central.s info post get cur_diagram_factor  \
		    $cur_res $cur_anal $cur_step ]
	    set restcommand "LineDiagram VectorDiagram cur_res $value"
	}
	"Scalar_Line_Diagram" {
	    set value [.gid.central.s info post get cur_diagram_factor \
		    $cur_res $cur_comp $cur_anal $cur_step ]
	    set restcommand "LineDiagram ScalarDiagram cur_res $cur_comp $value"
	}
	default {
	}
    }
    .central.s process escape escape escape escape Results AnalysisSel $newa \
	    escape escape

    set restypes [.central.s info postprocess get results_list Contour_Fill \
		      $newa 1]

    # to recreate the Results menu
    catch { unset ::BarresPriv(problemtype) }

    regsub {_[(].*[)]$} $cur_res {} cur_res_short
    set ipos [lsearch $restypes $cur_res_short*]
    if { $ipos == -1 } {
	regsub {_[^_]*$} $cur_res_short {} cur_res_short2
	set ipos [lsearch $restypes $cur_res_short2*]
	if { $ipos == -1 } {
	    #WarnWin "Result '$cur_res_short' is not contained in load case '$newa'"
	    StopResults
	    return
	} else {
	    set cur_res_short $cur_res_short2
	}
    }
    if { $restcommand != "" } {
	if { $cur_res_view == "Vector_Line_Diagram" } {
	     regsub -all {[[:<:]]cur_res[[:>:]]} $restcommand $cur_res_short restcommand
	    eval DisplayVectorLineDiagram [lrange $restcommand 2 end]
	} else {
    set cur_res [lindex $restypes $ipos]
    regsub -all {[[:<:]]cur_res[[:>:]]} $restcommand $cur_res restcommand
	eval .central.s process escape escape escape escape Results $restcommand \
		escape escape
	}
    }
}

#  proc AssignMaterialsFromLayers {} {

#      if { [lindex [.central.s info Mesh] 0] == 0 } { 
#  	WarnWin "error: This function can only be applied after generating the mesh."
#  	return
#      }
#      set usingnow [lindex [.central.s info Project] 4]
#      if { $usingnow != "GEOMETRYUSE" && $usingnow != "MESHUSE" } { return }

#      set DisableGraphics [.central.s disable graphics]
#      set DisableWarnLine [.central.s disable warnline]

#      .central.s disable graphics 1
#      .central.s disable warnline 1

#      if { $usingnow == "GEOMETRYUSE" } {
#  	.central.s process escape escape escape escape Meshing MeshView
#      }
#      .central.s process escape escape escape escape data materials
#      foreach i [.central.s info materials] {
#  	.central.s process DeleteMaterial $i
#      }
#      .central.s process escape escape escape escape data
#      foreach i [.central.s info layers] {
#  	.central.s process defaults BaseMaterial $i 1 name 
#  	.central.s process $i
#      }
#      .central.s process escape escape escape escape data Materials
#      foreach i [.central.s info layers] {
#  	.central.s process AssignMaterial $i layer:$i escape
#      }
#      if { $usingnow == "GEOMETRYUSE" } {
#  	.central.s process escape escape escape escape Geometry escape
#      }

#      if { !$DisableGraphics } { .central.s disable graphics 0 }
#      if { !$DisableWarnLine } { .central.s disable warnline 0 }

#      WarnWin "Now on, every layer will have a different set in the postprocessing part."
#  }
