#############################################################
# RATFINK v0.9
# A library of RTF output utilities
# $Id: rtflib.tcl,v 1.3 2002/06/13 18:37:55 ramsan Exp $
# Created 4 Dec 1995 / Last updated $Date: 2002/06/13 18:37:55 $
#############################################################
#
# Copyright (C) 1996 Joe English
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted.
#
# THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR
# IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
# WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
#
#############################################################
# RATFINK lives at <URL:http://www.art.com/cost/ratfink/>
#############################################################

proc rtf:warning {text} {
    #WarnWinText "RTF warning: $text"
    #puts stderr "RTF warning: $text"
}

###
### Character formatting attributes
###

array set rtf:CharAttrs {
    Font	{table rtf:FontTable "f"}
    FontSize	{halfpts "fs"}
    Bold	{boolean "b"}
    Italic	{boolean "i"}
    Hidden	{boolean "v"}

    AllCaps	{boolean "caps"}
    SmallCaps	{boolean "scaps"}
    StrikeThru	{boolean "strike"}

    Underline {enum {
	{0 ulnone} {None ulnone}
	{1 ul} {Continuous ul} {Single ul}
	{Dot uld}
	{Double uldb}
	{Word ulw}
    }}

    Subscript		{flag "sub"}
    Superscript		{flag "super"}
    NoSuperSub		{flag "nosupersub"}
    ShiftDown		{halfpts "dn"}
    ShiftUp		{halfpts "up"}

    FGColor		{table rtf:ColorTable "cf"}
    BGColor		{table rtf:ColorTable "cb"}

    LetterSpace		{dimension "expndtw"}
    KernAbove		{halfpts "kerning"}

    TextDirection {enum {
	{RTL	"rtlch"}
	{LTR	"ltrch"}
    }}
    Charset	{table rtf:CharsetTable  "cchs"}
    Language	{table rtf:LanguageTable "lang"}
}

###
### Paragraph formatting attributes
###
array set rtf:ParaAttrs {
    TabStops    	{proc rtf:ExpandTabStops}
    FirstIndent 	{dimension "fi"}
    LeftIndent  	{dimension "li"}
    RightIndent 	{dimension "ri"}
    SpaceBefore 	{dimension "sb"}
    SpaceAfter  	{dimension "sa"}
    LineSpacing 	{dimension "sl"}
    KeepTogether	{flag "keep"}
    KeepWithNext	{flag "keepn"}
    PageBreakBefore	{flag "pagebb"}
    Hyphenate   	{boolean "hyphpar"}
    Quadding {enum {
	    {Left	"ql"}
	    {Right	"qr"}
	    {Justify	"qj"}
	    {Center	"qc"}
    }}
}
# All character attributes are also paragraph attributes:
array set rtf:ParaAttrs [array get rtf:CharAttrs]

# Add attributes for rules:
array set rtf:ParaAttrs {
    InnerBorders	{flag "brdrbtw"}
    Box 		{proc rtf:ExpandRuleStyle "box"}
    TopBorder		{proc rtf:ExpandRuleStyle "brdrt"}
    BottomBorder	{proc rtf:ExpandRuleStyle "brdrb"}
    LeftBorder		{proc rtf:ExpandRuleStyle "brdrl"}
    RightBorder		{proc rtf:ExpandRuleStyle "brdrr"}
    OutsideBorder	{proc rtf:ExpandRuleStyle "brdrbar"}
}

###
### Section formatting attributes
###
array set rtf:SectAttrs {
    PageWidth    	{dimension "pgwsxn"}
    PageHeight    	{dimension "pghsxn"}
    LeftMargin    	{dimension "marglsxn"}
    RightMargin    	{dimension "margrsxn"}
    TopMargin    	{dimension "margtsxn"}
    BottomMargin	{dimension "margbsxn"}
    GutterWidth        	{dimension "guttersxn"}
    Landscape		{flag "lndscpsxn"}
    HasTitlePage	{flag "titlepg"}
    HeaderPosition	{dimension "headery"}
    FooterPosition	{dimension "footery"}
    SectionBreak {enum {
	    {None	"sbknone"}
	    {Page	"sbkpage"}
	    {EvenPage	"sbkeven"}
	    {OddPage	"sbkodd"}
    }}
    VAlign {enum {
	    {Top	"vertalt"}
	    {Bottom	"vertalb"}
	    {Middle	"vertalc"} {Center	"vertalc"}
	    {Justify	"vertalj"}
    }}
    FirstPageNumber 	{integer starts}
    RestartPageNumbers {enum {
	{0 "pgncont"}
    	{1 "pgnrestart"}
    }}
    PageNumbering {enum {
	{Arabic 	"pgndec"} {Decimal "pgndec"}
	{UCRoman	"pgnucrm"}
	{LCRoman	"pgnlcrm"}
	{UCAlpha	"pgnucltr"}
	{LCAlpha	"pgnlcltr"}
    }}

    Columns		{integer "cols"}
    ColumnSepSpace	{dimension "colsx"}
    ColumnLine		{flag "linebetcol"}
    ColumnDirection	{enum {
	{RTL "rtlsect"}
	{LTR "ltrsect"}
    }}
}

###
### Table attributes:
###

# nb: CellPosition is really a dimension
array set rtf:TableAttrs {
    CellPosition	{integer "cellx"}
    RowGap		{dimension "trgaph"}
    RowHeight		{dimension "trrh"}
    HeadingRow		{flag "trhdr"}
    Align		{enum {
			    {Left	"trql"}
			    {Right	"trqr"}
			    {Center	"trqc"}
			}}
    CellBorderBottom	{table rtf_RuleTable "clbrdrb"}
    CellBorderTop	{table rtf_RuleTable "clbrdrt"}
    CellBorderLeft	{table rtf_RuleTable "clbrdrl"}
    CellBorderRight	{table rtf_RuleTable "clbrdrr"}
}

global rtf_tableSettings rtf_tableDefaults
array set rtf_tableDefaults {
    width	6in
    numcols	1
    numcells	1
    rowgap	0pt
    rowsep	-
    colsep	-
    frame	-
    toprule	-
    botrule	-
    rowalign	Center
}
array set rtf_tableSettings [array get rtf_tableDefaults]

###
### Style attribute handling
###

proc rtf:ExpandAttributes {tblName attlist} {
    upvar #0 $tblName table
    set result ""
    while {[llength $attlist] >= 2} {
	set param [lindex $attlist 0]
	set value [lindex $attlist 1]
	set attlist [lrange $attlist 2 end]

	if ![info exists table($param)] {
	    rtf:warning "No parameter $param in $tblName"
	    continue
	}

	set key $table($param)

	switch [lindex $key 0] {
	    unknown {
		rtf:warning "Don't grok parameter $param=$value in $tblName"
		continue
	    }

	    boolean {
		set csname [lindex $key 1]
		if $value {
		    set cs "\\$csname"
		} else {
		    set cs "\\${csname}0"
		}
	    }

	    flag {
		set csname [lindex $key 1]
		if $value {
		    set cs "\\$csname"
		} else {
		    rtf:warning "Parameter $param can only be turned on"
		}
	    }

	    integer {
		if ![regexp -- {^-?[0-9]+$} $value] {
		    rtf:warning "bad integer $value in parameter $param"
		    continue
		}
		set csname [lindex $key 1]
		set cs "\\${csname}$value"
	    }

	    dimension {
		set csname [lindex $key 1]
		set cs "\\${csname}[rtf:cvTwips $value]"
	    }

	    halfpts {
		set csname [lindex $key 1]
		set cs "\\${csname}[rtf:cvHalfpts $value]"
	    }

	    enum {
		if { [info exists cs] } {
		    unset cs
		}
		set enumnames {}
		foreach enumpair [lindex $key 1] {
		    set enumname [lindex $enumpair 0]
		    if [string match $value $enumname] {
			set cs "\\[lindex $enumpair 1]";
			break;
		    }
		    lappend enumnames $enumname;
		}
		if ![info exists cs] {
		    rtf:warning \
		"Unknown value '$param' '$value': must be one of $enumnames"
		    continue
		}
	    }

	    table {
		set subtblName [lindex $key 1]
		upvar #0 $subtblName subtable
		if ![info exists subtable($value)] {
		    rtf:warning "$param: no mapping for $value in $subtblName"
		    continue
		}
		if {[llength $key] == 3} {
		    set cs "\\[lindex $key 2]$subtable($value)"
		} else {
		    set cs $subtable($value)
		}
	    }

	    proc {
		set procName [lindex $key 1]
		set cs [$procName $value]
		if {[llength $key] == 3} {
		    set cs "\\[lindex $key 2]$cs"
		}
	    }

	    pseudo {
		# pseudo-argument; ignore
		set cs ""
	    }

	    default {
		rtf:warning "Bad parameter type [lindex $key 0] for $param"
		continue
	    }
	}
	append result $cs
    }
    if {[llength $attlist] != 0} {
	rtf:warning "Leftover arguments: <$attlist>"
    }
    return $result
}

# Expand attributes from associative array.
# 'paramlist' if present specifies the desired output order.
#
proc rtf:ExpandArray {tblName specsName {paramlist {}}} {
    upvar #0 $tblName table
    upvar 1 $specsName specs

    set speclist {}
    if {$paramlist == {}} { set paramlist [array names specs] }
    foreach param $paramlist {
	if [info exists specs($param)] {
	    lappend speclist $param $specs($param)
	}
    }
    # error-check:
    foreach param [array names specs] {
	if ![info exists table($param)] {
	    rtf:warning "No parameter $param in $tblName"
	}
    }

    return [rtf:ExpandAttributes $tblName $speclist]
}

###
### Stylesheet management
###

global rtf_styleSheet; 	# key: $styleName,(DESC|NUM|DEF|TYPE|BASEDON)
global rtf_styleNames;	set rtf_styleNames {}
global rtf_styleNum; 	set rtf_styleNum 1

proc rtf:paraStyle {name desc args} { rtf:defineStyle PARA $name $desc $args }
proc rtf:charStyle {name desc args} { rtf:defineStyle CHAR $name $desc $args }
proc rtf:sectStyle {name desc args} { rtf:defineStyle SECT $name $desc $args }

proc rtf:defineStyle {type name desc arglist} {
    global rtf_styleNum rtf_styleSheet rtf_styleNames

    set rtf_styleSheet($name,NUM) $rtf_styleNum
    set rtf_styleSheet($name,DESC) $desc
    set rtf_styleSheet($name,TYPE) $type
    set styledef ""

    incr rtf_styleNum
    lappend rtf_styleNames $name

    while {[string match "-*" [lindex $arglist 0]]} {
	set option [lindex $arglist 0]
	set value [lindex $arglist 1]
	set arglist [lrange $arglist 2 end]
	switch -- $option {
	    -basedon {
		if ![info exists rtf_styleSheet($value,DEF)] {
		    rtf:warning "$name: style $value not defined"
		    continue
		}
		if {$rtf_styleSheet($value,TYPE) != $type} {
		    rtf:warning "$name based on style of different type"
		    continue
		}
		set rtf_styleSheet($name,BASEDON) $value
		append styledef "$rtf_styleSheet($value,DEF)"
	    }
	    default {
		rtf:warning "rtf:defineStyle -- bad option $option"
	    }
	}
    }
    set attlist [lindex $arglist 0]
    switch $type {
	PARA { set styletable rtf:ParaAttrs }
	CHAR { set styletable rtf:CharAttrs }
	SECT { set styletable rtf:SectAttrs }
    }
    set stylespec [rtf:ExpandAttributes $styletable $attlist]
    append styledef $stylespec

    set rtf_styleSheet($name,DEF) $styledef
    return;
}

proc rtf:writeStyleSheet {} {
    global rtf_styleNames rtf_styleSheet
    rtf:bgroup
    rtf:write "\\stylesheet"
    foreach style $rtf_styleNames {
	rtf:bgroup
	switch $rtf_styleSheet($style,TYPE) {
	    PARA { set styletype "\\s" }
	    CHAR { set styletype "\\cs" }
	    SECT { set styletype "\\ds" }
	}
	rtf:write "$styletype$rtf_styleSheet($style,NUM)"
	rtf:write "$rtf_styleSheet($style,DEF)"
	if [info exists rtf_styleSheet($style,BASEDON)] {
	    set basedon $rtf_styleSheet($style,BASEDON)
	    rtf:write "\\sbasedon$rtf_styleSheet($basedon,NUM)"
	}
	rtf:write " $rtf_styleSheet($style,DESC);"
	rtf:egroup
    }
    rtf:egroup
}

###
### Dimensions
###
global rtf_twipsMap
array set rtf_twipsMap "
	twip	1
	pt	20
	halfpt	10
	in	[expr 72 * 20]
	pc	[expr 12 * 20]
	pica	[expr 12 * 20]
	cm	567
	mm	56.7
"

# %%% The conversion factor 56.7 for 'mm' was determined --
# %%% by experimentation -- so RATFINK's definition of A4 paper (210mm x 297mm)
# %%% matches WFW95v7's definition (\paperw11907\paperh16840)
# %%% after rounding.  [expr (72 * 20) / 25.4] == 56.6929
# %%% is a *little* bit smaller than what Word appears to use.

proc rtf:cvTwips {dim} {
    global rtf_twipsMap
    if ![regexp -- {^(-?[0-9\.]+)([a-z]+)$} $dim dummy qty units] {
	rtf:warning "Bad dimension $dim"
	return 0
    }
    if ![info exists rtf_twipsMap($units)] {
	rtf:warning "Bad units $units"
	return 0
    }
    set scale $rtf_twipsMap($units)
    return [expr round($qty * $scale)]
}

proc rtf:cvHalfpts {dim} {
    if [regexp -- {^-?([0-9\.]+)$} $dim] {
	return $dim
    }
    return [expr round([rtf:cvTwips $dim] / 10)]
}

###
### Fonts
###
global rtf_fonttbl
set rtf_fonttbl [join {
    {{\f0\froman\fcharset0\fprq2 Times New Roman;}}
    {{\f1\fswiss\fcharset0\fprq2 Arial;}}
    {{\f2\fmodern\fcharset0\fprq1 Courier New;}}
    {{\f3\froman\fprq2\fcharset2 Symbol;}}
    } "" ]
array set rtf:FontTable {
    roman	0
    sans	1
    mono	2
    symbol   	3   
}

proc rtf:writeFontTable {} {
    #+%%%
    global rtf_fonttbl;
    rtf:bgroup
    rtf:write "\\fonttbl $rtf_fonttbl"
    rtf:egroup
}

###
### Colors
###

array set rtf_ColorAttrs {
    Red {integer "red" }
    Green {integer "green" }
    Blue {integer "blue" }
}

proc rtf:colorStyle {name attlist} {
    global rtf_ColorTableDefs rtf:ColorTable
    if [info exists rtf:ColorTable($name)] {
	rtf:warning "Redefining color style $name"
	set inum [set rtf:ColorTable($name)]
    } elseif { ![info exists  rtf_ColorTableDefs] } {
	set inum 1
    } else {
	set inum [lindex [lsort -integer [array names rtf_ColorTableDefs]] end]
	incr inum
    }
    set rtf_ColorTableDefs($inum) [rtf:ExpandColorStyle $attlist]
    set rtf:ColorTable($name) $inum
}
proc rtf:ExpandColorStyle {attlist} {
    global rtf_ColorAttrs rtf_ColorTable
    if {[llength $attlist] == 1} {
	global rtf_ColorTable
	if [info exists rtf_ColorTable($attlist)] {
	    return $rtf_ColorTable($attlist)
	} else {
	    rtf:warning "No color style <$attlist> defined"
	    return ""
	}
    }
    set tmp(Red) 0
    set tmp(Green) 0
    set tmp(Blue) 0
    array set tmp $attlist
    return [rtf:ExpandArray rtf_ColorAttrs tmp "Red Green Blue"]
}

proc rtf:writeColorTable {} {
    global rtf_ColorTableDefs
    set inum 1
    rtf:bgroup
    rtf:write "\\colortbl ;"
    foreach i [lsort -integer [array names rtf_ColorTableDefs]] {
	rtf:write "$rtf_ColorTableDefs($i);"
    }
    rtf:egroup
}

# Pre-defined color styles:
rtf:colorStyle red	{ Red 255 Green 0 Blue 0 }
rtf:colorStyle blue 	{ Red 0 Green 0 Blue 255 }
rtf:colorStyle green 	{ Red 0 Green 255 Blue 0 }

   

###
### Tab stops
###

# The order of the control words must be <kind>? <leaders>? <position>,
array set rtf_TabAttrs {
    Align {enum {
	{Left	"tql"}
	{Right	"tqr"}
	{Center	"tqc"}
	{Decimal "tqdec"}
    }}
    Leaders {enum {
	{Dot	"tldot"}
	{Hyphen	"tlhyph"}
	{Under	"tlul"}
	{Thick	"tlth"}
	{Equal	"tleq"}
    }}
    Position {dimension "tx"}
}
proc rtf:tabStops {name stops} {
    global rtf_TabTable
    if [info exists rtf_TabTable($name)] {
	rtf:warning "Redefining tab settings $name"
    }
    set rtf_TabTable($name) [rtf:ExpandTabStops $stops]
}

proc rtf:ExpandTabStops {stops} {
    global rtf_TabAttrs rtf_TabTable
    set result ""
    if [regexp {^[a-zA-Z][-_a-zA-Z0-9]*$} $stops] {
	if [info exists rtf_TabTable($stops)] {
	    return $rtf_TabTable($stops)
	} else {
	    rtf:warning "Tab settings <$stops> not defined"
	    return ""
	}
    }
    foreach stop $stops {
	catch {unset tmp}
	set tmp(Position) [lindex $stop 0]
	array set tmp [lrange $stop 1 end];
	append result \
		[rtf:ExpandArray rtf_TabAttrs tmp "Align Leaders Position"]
    }
    return $result
}

###
### Rule styles
###

# Order of control words is:
# <brdrk>, <width>?, <space>?, <color>?
# <brdrk> == 'Style' is mandatory and must be first.
#
array set rtf_RuleAttrs {
    Style {enum {
	{Normal	"brdrs"}
	{Thick	"brdrth"}
	{Double	"brdrdb"}
	{Shadow	"brdrsh"}
	{Dot	"brdrdot"}
	{Dash	"brdrdash"}
	{Hairline	"brdrhair"}
    }}
    Margin	{dimension "brsp"}
    Thickness	{dimension "brdrw"}
    Color	{table rtf:colorTable "brdrcf"}
}
proc rtf:ruleStyle {name attlist} {
    global rtf_RuleTable
    if [info exists rtf_RuleTable($name)] {
	rtf:warning "Redefining rule style $name"
    }
    set rtf_RuleTable($name) [rtf:ExpandRuleStyle $attlist]
}
proc rtf:ExpandRuleStyle {attlist} {
    global rtf_RuleAttrs rtf_RuleTable
    if {[llength $attlist] == 1} {
	global rtf_RuleTable
	if [info exists rtf_RuleTable($attlist)] {
	    return $rtf_RuleTable($attlist)
	} else {
	    rtf:warning "No rule style <$attlist> defined"
	    return ""
	}
    }
    set tmp(Style) Normal;	# 'Style' is mandatory; set default
    array set tmp $attlist
    return [rtf:ExpandArray rtf_RuleAttrs tmp "Style Margin Thickness Color"]
}
# Pre-defined rule styles:
rtf:ruleStyle thin	{ Style Normal  Thickness 0.75pt }
rtf:ruleStyle thick 	{ Style Normal  Thickness 1.50pt }
rtf:ruleStyle double 	{ Style Double  Thickness 0.75pt }

###
### Document-wide attributes
###
array set rtf:DocAttrs {

    PaperWidth		{dimension "paperw"}
    PaperHeight		{dimension "paperh"}
    LeftMargin		{dimension "margl"}
    RightMargin		{dimension "margr"}
    TopMargin		{dimension "margt"}
    BottomMargin	{dimension "margb"}
    DefaultTabWidth	{dimension "deftab"}

    TwoSide		{flag "facingp"}
    MirrorMargins	{flag "margmirror"}
    Landscape		{flag "landscape"}
    FirstPageNumber	{integer "pgnstart"}
    WidowControl	{flag "widowctrl"}
    GutterWidth		{dimension "gutter"}

    SaveAsRTF			{flag "defformat"}
    Protection	{enum {
    	{Forms   	"formprot"}
    	{Revisions	"revisions\\revprot"}
    	{Annotations	"annotprot"}
    	{AllProtected	"allprot"}
    }}

    Hyphenate			{boolean "hyphauto"}
    HyphenationHotZone		{dimension "hyphhotz"}
    HyphenationLadderCount	{integer "hyphconsec"}
    HyphenateAllCaps		{boolean "hyphcaps"}

    FootnoteNumbering {enum {
    	{Arabic 	"ftnnar"}
    	{LCAlpha	"ftnnalc"}
    	{UCAlpha	"ftnnauc"}
    	{LCRoman	"ftnnrlc"}
    	{UCRoman	"ftnnruc"}
    	{Chicago	"ftnnchi"}
    }}
    FootnoteLocation {enum {
	{EndOfSection	"endnotes"}
	{EndOfDocument	"enddoc"}
    }}
    FootnotePlacement {enum {
    	{PageBottom	"ftnbj"}
    	{BeneathText	"ftntj"}
    }}
    FirstFootnoteNumber	{integer "ftnstart"}
    FootnoteRestart {enum {
	{Continuous	"ftnrstcont"}
	{AtSection	"ftnrestart"}
    	{AtPage		"ftnrstpg"}
    }}
}
# Add 'papersize' document options:
set tmp {}; foreach psz {
	{ A4    	210mm	297mm	}
	{ A5    	148mm	210mm	}
	{ B5    	176mm	250mm	}
	{ Letter	8.5in	11in 	}
	{ Legal 	8.5in	14in 	}
	{ Executive	7.25in	10.5in	}
} {
    set pszn [lindex $psz 0]
    set pszw [rtf:cvTwips [lindex $psz 1]]
    set pszh [rtf:cvTwips [lindex $psz 2]]
    lappend tmp [list $pszn "paperw$pszw\\paperh$pszh"]
}
set rtf:DocAttrs(PaperSize) "enum { $tmp }"

proc rtf:documentFormat {params} {
    global rtf_DocumentFormat
    array set rtf_DocumentFormat $params
}
proc rtf:writeDocumentFormat {} {
    global rtf_DocumentFormat
    if [info exists rtf_DocumentFormat] {
	rtf:write [rtf:ExpandArray rtf:DocAttrs rtf_DocumentFormat]
	rtf:write " "
    }
}

###
### Information group
###
proc rtf:writeInformationGroup {} {
    # %%% fake it for now...
    rtf:bgroup
    rtf:write "\\info"
	rtf:bgroup
	rtf:write "\\doccomm Created by RATFINK/Cost"
	rtf:egroup
    rtf:egroup
}

###
### Output
###
global rtf_fp
set rtf_fp stdout

proc rtf:write {text} {
    global rtf_fp
    puts $rtf_fp $text nonewline
}
proc rtf:bgroup {} {
    rtf:write \{
}
proc rtf:egroup {} {
    rtf:write \}
}

if [array exists COST] {
    substitution rtf:Escape {
	"{"	"\\{"
	"}"	"\\}"
	"\\"	"\\\\"
	"\t"	" "
	"\n"	" "
	"`"	"\\lquote "
	"'"	"\\rquote "
	"``"	"\\ldblquote "
	"''"	"\\rdblquote "
	"--"	"\\endash "
	"---"	"\\emdash "
    }
    substitution rtf:EscapeLineSpecific {
	"{"	"\\{"
	"}"	"\\}"
	"\\"	"\\\\"
	"\t"	"\\tab "
	"\n"	"\\line\n"
    }
} else {
    proc rtf:Escape {text} {
	regsub -all {[{}\\]} $text {\\&} text
	return $text
    }
}
proc rtf:text {text} {
    global rtf_state
    if !$rtf_state(inpara) {rtf:startPara [rtf:currentParaStyle]}
    rtf:write [rtf:Escape $text]
}
proc rtf:insert {data} {
    global rtf_state
    if !$rtf_state(inpara) {rtf:startPara [rtf:currentParaStyle]}
    rtf:write $data
}

# may be overridden by applications:
proc rtf:currentParaStyle {} {
    #rtf:warning "No current paragraph style defined"
    return "normal"
}

array set rtfSpecial {
    Tab			"\\tab "
    LineBreak		"\\line "
    PageBreak		"\\page "
    ColumnBreak		"\\column "

    EmDash		"\\emdash "
    EnDash		"\\endash "
    EmSpace		"{\\emspace  }"
    EnSpace		"{\\enspace  }"
    Bullet		"\\bullet "

    LSQuote		"\\lquote "
    RSQuote		"\\rquote "
    LDQuote		"\\ldblquote "
    RDQuote		"\\rdblquote "

    PageNumber		"\\chpgn "
    SectionNumber	"\\sectnum "
    FootnoteNumber	"\\chftn "
}
#    Ellipsis		"{\\expndtw20 ...}" ???
# There is reportedly an ellipsis character in the Symbol character set...
#

proc rtf:special {code} {
    global rtfSpecial
    if [info exists rtfSpecial($code)] {
	rtf:insert $rtfSpecial($code)
    } else {
	rtf:warning "Unrecognized special <$code>"
    }
}

proc rtf:tab {} 	{ rtf:write "\\tab " }
proc rtf:lineBreak {} 	{ rtf:write "\\line " }
proc rtf:pageBreak {} 	{ rtf:write "\\page " }
proc rtf:columnBreak {}	{ rtf:write "\\column " }

###
### Document structure
###

global rtf_state
array set rtf_state {
    inpara	0
    insection	0
    intable	0
    inrow	0
    incell	0
    cellno	0
    diversion	""
}

proc rtf:start {} {
    rtf:bgroup
# Header:
    rtf:write "\\rtf1\\ansi"
    rtf:write "\\deff0"
    rtf:writeFontTable
    #- fileTable
    rtf:writeColorTable
    rtf:writeStyleSheet
    #- revisionTable
# Document:
    rtf:writeInformationGroup
    rtf:writeDocumentFormat
# section text follows...
}
proc rtf:end {} {
    rtf:egroup
}

proc rtf:startSection {{style -}} {
    global rtf_state rtf_styleSheet
    rtf:endSection
    rtf:write "\\sectd"
    if {$style != "-"} {
	if {![info exists rtf_styleSheet($style,DEF)] \
		|| $rtf_styleSheet($style,TYPE) != "SECT"} {
	    rtf:warning "Style <$style> not a section style"
	} else {
	    rtf:write $rtf_styleSheet($style,DEF)
	}
	rtf:write " "
    }
    set rtf_state(insection) 1
}
proc rtf:endSection {} {
    global rtf_state
    if $rtf_state(insection) {
	rtf:write "\\sect "
    }
    set rtf_state(insection) 0
}

proc rtf:startPara {style} {
    global rtf_styleSheet rtf_state
    if $rtf_state(inpara) { rtf:write "\\par" }
    rtf:write "\\pard\\plain "
    if $rtf_state(intable) { rtf:write "\\intbl " }
    if [info exists rtf_styleSheet($style,DEF)] {
	if {$rtf_styleSheet($style,TYPE) != "PARA"} {
	    rtf:warning "$style not a paragraph style"
	} else {
	    rtf:write "\\s$rtf_styleSheet($style,NUM)"
	}
	rtf:write "$rtf_styleSheet($style,DEF) "
    } elseif {$style != "-"} {
	rtf:warning "No definition for <$style> style"
    }
    set rtf_state(inpara) 1
}
proc rtf:endPara {} {
    global rtf_state
    if $rtf_state(inpara) {
	rtf:write "\\par\n"
    }
    set rtf_state(inpara) 0
}

proc rtf:startPhrase {style} {
    rtf:bgroup
    rtf:setCharStyle $style
}
proc rtf:endPhrase {} {
    rtf:egroup
}

proc rtf:setCharStyle {style} {
    global rtf_styleSheet rtf_state
    if !$rtf_state(inpara) { rtf:startPara [rtf:currentParaStyle] }
    if [info exists rtf_styleSheet($style,DEF)] {
	if {$rtf_styleSheet($style,TYPE) != "CHAR"} {
	    rtf:warning "$style not a character style"
	} else {
	    rtf:write "\\cs$rtf_styleSheet($style,NUM)"
	}
	rtf:write "$rtf_styleSheet($style,DEF) "
    } else {
	rtf:warning "No definition for character style <$style>"
    }
}

###
### Tables
###

#
# Different ways to specify number and widths of columns:
#
#  + number of columns and total width (equal-sized columns)
#  + list of relative widths and total width
#  + list of absolute widths
# 
proc _rtf:tableRelativeWidths {colwidths} {
    global rtf_tableSettings rtf_cellPositions
    set tablewidth [rtf:cvTwips $rtf_tableSettings(width)]
    set sumwidths 0
    foreach colwidth $colwidths { incr sumwidths $colwidth }
    set colend 0
    set colnum 0
    foreach colwidth $colwidths {
	incr colend [expr round(($tablewidth*$colwidth)/$sumwidths)]
	incr colnum
	set rtf_cellPositions($colnum) $colend
    }
    set rtf_tableSettings(numcols) $colnum
}

proc _rtf:tableAbsoluteWidths {colwidths} {
    global rtf_tableSettings rtf_cellPositions
    set colend 0
    set colnum 0
    foreach colwidth $colwidths {
	incr colend [rtf:cvTwips $colwidth]
	incr colnum
	set rtf_cellPositions($colnum) $colend
    }
    set rtf_tableSettings(width) $colend
    set rtf_tableSettings(numcols) $colnum
}

proc _rtf:tableEqualWidths {numcols} {
    global rtf_tableSettings rtf_cellPositions
    set rtf_tableSettings(numcols) $numcols
    set tablewidth [rtf:cvTwips $rtf_tableSettings(width)]
    set colnum 0
    while {$colnum < $numcols} {
	incr colnum
	set rtf_cellPositions($colnum) \
		[expr round($tablewidth*$colnum/$numcols)]
    }
}

proc rtf:startTable {args} {
    global rtf_state rtf_tableSettings rtf_tableDefaults
    if $rtf_state(intable) {
	rtf:warning "Already in table"
	return;
    }
    array set rtf_tableSettings [array get rtf_tableDefaults]
    catch {unset rtf_cellPositions}
    set option ""
    foreach arg $args {
	if {$option == ""} { set option $arg; continue; }
	switch -- $option {
	    -width	{ set rtf_tableSettings(width) $arg }
	    -align	{ set rtf_tableSettings(rowalign) $arg }
	    -numcols 	{ _rtf:tableEqualWidths $arg }
	    -relwidths	{ _rtf:tableRelativeWidths $arg }
	    -abswidths	{ _rtf:tableAbsoluteWidths $arg }
	    -toprule 	{ set rtf_tableSettings(toprule) $arg }
	    -colsep 	{ set rtf_tableSettings(colsep) $arg }
	    -rowsep 	{ set rtf_tableSettings(rowsep) $arg }
	    -frame 	{ set rtf_tableSettings(frame) $arg
			  set rtf_tableSettings(toprule) $arg }
	    default { rtf:warning "rtf:startTable: bad option $option" }
	}
	set option ""
    }
    if {$option != ""} {
	rtf:warning "rtf:startTable: no argument for option $option"
    }
    set rtf_tableSettings(botrule) $rtf_tableSettings(rowsep)
    array set rtf_state { intable 1  inrow 0  incell 0  cellno 0 }
}

proc rtf:startRow {args} {
    global rtf_state rtf_tableSettings

    if !$rtf_state(intable) {
	rtf:warning "Not in a table"
    }
    if $rtf_state(inrow) rtf:endRow

    set isheading 0
    set colspans ""
    set option ""
    foreach arg $args {
	if {$option == ""} { set option $arg; continue; }
	switch -- $option {
	    -colspans	{ set colspans $arg }
	    -toprule 	{ set rtf_tableSettings(toprule) $arg }
	    -botrule 	{ set rtf_tableSettings(botrule) $arg }
	    -colsep 	{ set rtf_tableSettings(colsep) $arg }
	    -heading	{ set isheading $arg }
	    default	{
		rtf:warning "rtf:startRow: bad option $option" 
	    }
	}
	set option ""
    }
    if {$option != ""} {
	rtf:warning "rtf:startRow: no argument for option $option"
    }

    set rowatts [list \
	RowGap	$rtf_tableSettings(rowgap) \
	Align	$rtf_tableSettings(rowalign) ]
    if $isheading {
	lappend rowatts HeadingRow $isheading
    }
    rtf:write "\\trowd[rtf:ExpandAttributes rtf:TableAttrs $rowatts]\n"

    if {$colspans != ""} {
	set rtf_tableSettings(numcells) [llength $colspans]
	set colno 1
	foreach span $colspans {
	    rtf:cellDef $colno $span
	    incr colno $span
	}
    } else {
	set rtf_tableSettings(numcells) $rtf_tableSettings(numcols)
	for {set colno 1} {$colno <= $rtf_tableSettings(numcols)} {incr colno} {
	    rtf:cellDef $colno
	}
    }
    rtf:write "\n"
    array set rtf_state { inrow 1 incell 0 cellno 0 }
}

proc rtf:cellDef {colno {span 1}} {
    global rtf_tableSettings rtf_cellPositions
    set start $colno
    set end [expr $colno + $span - 1]
    if {$end > $rtf_tableSettings(numcols)} {
	rtf:warning "Cell spans too many columns"
	return
    }
    if {$start == 1} {
	set border(Left) $rtf_tableSettings(frame) 
    } else {
	set border(Left) $rtf_tableSettings(colsep) 
    }
    if {$end == $rtf_tableSettings(numcols)} {
	set border(Right) $rtf_tableSettings(frame) 
    } else { 
	set border(Right) $rtf_tableSettings(colsep)
    }
    set border(Top)   	$rtf_tableSettings(toprule)
    set border(Bottom)	$rtf_tableSettings(botrule)
    set cellAtts {}
    foreach b {Top Bottom Left Right} {
	if {$border($b) != "-"} {
	    lappend cellAtts "CellBorder$b" $border($b)
	}
    }
    # cellx must be last
    lappend cellAtts CellPosition $rtf_cellPositions($end)
    rtf:write [rtf:ExpandAttributes rtf:TableAttrs $cellAtts]
}

# ... need to distinguish between cells which contain paragraphs
# ... and those which only contain inline stuff;
# ... for the latter apply paragraph properties when the cell starts

proc rtf:startCell {{style -}} {
    global rtf_state
    if {$rtf_state(incell)} { rtf:endCell }
    set rtf_state(incell) 1
    incr rtf_state(cellno)
    rtf:startPara $style
}
proc rtf:endCell {} {
    global rtf_state
    if !$rtf_state(incell) { rtf:warning "Not in a cell"; return }
    rtf:write "\\cell\n"
    array set rtf_state { inpara 0 incell 0 }
}

proc rtf:endRow {} {
    global rtf_state rtf_tableSettings
    if !$rtf_state(inrow) {
	rtf:warning "Not in a table row"
	return
    }
    if $rtf_state(incell) rtf:endCell
    if {$rtf_state(cellno) != $rtf_tableSettings(numcells)} {
	rtf:warning \
	    "$rtf_state(cellno) cells; should be $rtf_tableSettings(numcells)"
    }
    rtf:write "\\row\n"
    set rtf_state(inrow) 0
    set rtf_tableSettings(toprule) $rtf_tableSettings(botrule)
    set rtf_tableSettings(botrule) $rtf_tableSettings(rowsep)
}

proc rtf:endTable {} {
    global rtf_state
    if !$rtf_state(intable) {
	rtf:warning "Not in a table"
	return
    }
    if $rtf_state(inrow) rtf:endRow
    # without this, Word sometimes crashes...
    rtf:write "\\pard\\par\n"
    set rtf_state(intable) 0
}

###
### Bookmarks
###

proc rtf:startBookmark {name} {
    rtf:bgroup
    rtf:write "{\\*\\bkmkstart $name}"
    rtf:egroup
}

proc rtf:endBookmark {name} {
    rtf:bgroup
    rtf:write "{\\*\\bkmkend $name}"
    rtf:egroup
}

###
### Fields
###

proc rtf:startField {inst} {
    rtf:bgroup
	rtf:write "\\field"
	rtf:bgroup
	    rtf:write "\\*\\fldinst "
	    rtf:write [rtf:Escape $inst]
	rtf:egroup
	rtf:bgroup
	    rtf:write "\\fldrslt "
}
proc rtf:endField {} {
	rtf:egroup
    rtf:egroup
}
proc rtf:insertField {inst {rslt ""}} {
  rtf:write \
    "{\\field{\\*\\fldinst [rtf:Escape $inst]}{\\fldrslt [rtf:Escape $rslt]}}"
}
# %%% note: syntax productions in spec disagree with examples,
# %%% what Word actually does, and common sense.

###
### Destination groups
###
array set rtf_Destinations {
    Header		"header"
    LeftHeader		"headerl"
    RightHeader 	"headerr"
    FirstPageHeader	"headerf"
    Footer		"footer"
    FirstPageFooter	"footerf"
    LeftFooter  	"footerl"
    RightFooter  	"footerr"
    Footnote		"footnote"
    Endnote		"footnote\\ftnalt"
}
# %%% Should stack diversions, current state &c;
proc rtf:divert {diversion} {
    global rtf_state rtf_Destinations rtf_savedState
    if {$rtf_state(diversion) != ""} {
	rtf:warning "Diversion $diversion within $rtf_state(diversion)"
    }
    if ![info exists rtf_Destinations($diversion)] {
	rtf:warning "Bad destination $diversion"
	return
    }
    rtf:bgroup
    rtf:write "\\*\\$rtf_Destinations($diversion)"
    array set rtf_savedState [array get rtf_state]
    array set rtf_state {inpara 0 insection 0 incell 0 inrow 0 intable 0}
    set rtf_state(diversion) $diversion
    return
}

proc rtf:undivert {} {
    global rtf_state rtf_savedState
    if {$rtf_state(diversion) == ""} {
	rtf:warning "No current diversion"
	return
    }
    rtf:egroup
    array set rtf_state [array get rtf_savedState]
    unset rtf_savedState
    set rtf_state(diversion) ""
    return;
}

# RAMSAN
proc rtf:writeimage { img } {

#     package require Img
#     package require base64

#     binary scan [base64::decode [$img data -format jpeg]] H* jpg
#     rtf:write "\\f1{"
#     rtf:write {\pict\jpegblip}
#     rtf:write \n$jpg
#     rtf:write "}"
    
}

# RAMSAN
proc rtf:bullet {} {
    rtf:write "{"
    rtf:write {\pict{\*\picprop\shplid1027{\sp{\sn shapeType}{\sv 75}}{\sp{\sn fFlipH}{\sv 0}}{\sp{\sn fFlipV}{\sv 0}}{\sp{\sn pibName}}{\sp{\sn pibFlags}{\sv 2}}{\sp{\sn fLine}{\sv 0}}{\sp{\sn fIsBullet}{\sv 1}}{\sp{\sn fLayoutInCell}{\sv 1}}}\picscalex100\picscaley100\piccropl0\piccropr0\piccropt0\piccropb0
\picw318\pich318\picwgoal180\pichgoal180\pngblip\bliptag1511174400{\*\blipuid 5a12b10041955a9567c860bffb141933}89504e470d0a1a0a0000000d494844520000000c0000000c02030000002b1bb474000000017352474200aece1ce900000006504c5445c0c0c00000668f0665cb
0000000c636d50504a436d7030373132020000010c07da1b0000000174524e530040e6d8660000001c494441541819636040078c010c0cac210c0ca2a1080ce283c41918002c2d02930e4438520000000049454e44ae426082}
    rtf:write "}  "
}

proc rtf:writejpgimage { jpgfile } {
    set fin [open $jpgfile r]
    fconfigure $fin -translation binary
    binary scan [read $fin [file size $jpgfile]] H* jpg
    close $fin
    rtf:write "\\f1{"
    rtf:write {\pict\jpegblip}
    rtf:write \n$jpg
    rtf:write "}"
}

proc rtf:parsetag { tag } {

    set retlist ""
    regexp {^\s*([/\w]+)} $tag {} tagname
    lappend retlist [string tolower $tagname]
    foreach "{} name val valalt" [regexp -inline -all {(?i)(\w+)=(?:"([^"]*)"|([^"\s]+))} $tag] {
	if { $val == "" } { set val $valalt }
	lappend retlist [string tolower $name] $val
    }
    return $retlist
}

proc rtf:startParaR { style } {
    global ParaOn PhraseOn

    if { $PhraseOn } {
	rtf:endPhrase
	set PhraseOn 0
    }
    if { $ParaOn } {
	rtf:endPara
	set ParaOn 0
    }
    rtf:startPara $style
    set ParaOn 1
    rtf:ApplyPhrases
}

proc rtf:endParaR {} {
    global ParaOn PhraseOn

    if { $PhraseOn } {
	rtf:endPhrase
	set PhraseOn 0
    }
    if { $ParaOn } {
	rtf:endPara
	set ParaOn 0
    }
}

proc rtf:UnApplyPhrases {} {
    global PhraseOn

    if { $PhraseOn } {
	rtf:endPhrase
	set PhraseOn 0
    }
}

proc rtf:ApplyPhrases {} {
    global StyleState PhraseOn

    if { $PhraseOn } {
	rtf:endPhrase
	set PhraseOn 0
    }

    set color ""
    set face ""
    set strong 0

    foreach i $StyleState {
	switch [lindex $i 0] {
	    strong { set strong 1 }
	    font {
		foreach j [lrange $i 1 end] {
		    switch $j {
			red { set color red }
			symbol { set face symbol }
		    }
		}
	    }
	}
    }
    if { $face != "" } {
	if { $face != "symbol" } { rtf:warning "Warning: there is no style with face $face" }
	if { $strong } { rtf:warning "Warning: there is no style symbol and bold" }
	if { $color != "" } { rtf:warning "Warning: there is no style red and symbol" }
	rtf:startPhrase symbol
	set PhraseOn 1
    } elseif { $color != "" } {
	if { $color != "red" } { rtf:warning "Warning: there is no style with color $color" }
	if { $strong } {
	    rtf:startPhrase redbold
	} else {
	    rtf:startPhrase red
	}
	set PhraseOn 1
    } elseif { $strong } {
	rtf:startPhrase bold
	set PhraseOn 1
    }

}

proc rtf:writehtmltableFinal {} {
    global TableDef

    set ncolsmax 0
    for { set irow 1 } { $irow <= $TableDef(nrows) } { incr irow } {
	set ncols 0
	for { set icol 1 } { $icol <= $TableDef(ncols) } { incr icol } {
	    if { [info exists TableDef($irow,$icol,colspan)] } {
		incr ncols $TableDef($irow,$icol,colspan)
	    }
	}
	if { $ncols > $ncolsmax } { set ncolsmax $ncols }
    }
    rtf:startTable -numcols $ncolsmax -frame thin -align Center -rowsep thin \
	-colsep thin

    for { set irow 1 } { $irow <= $TableDef(nrows) } { incr irow } {
	set colspans ""
	set ncols 0
	for { set icol 1 } { $icol <= $TableDef(ncols) } { incr icol } {
	    if { [info exists TableDef($irow,$icol,colspan)] } {
		lappend colspans $TableDef($irow,$icol,colspan)
		incr ncols $TableDef($irow,$icol,colspan)
	    }
	}
	for { set icol $ncols } { $icol < $ncolsmax } { incr icol } {
	    lappend colspans 1
	}
	rtf:startRow -colspans $colspans
	set ncols 0
	for { set icol 1 } { $icol <= $TableDef(ncols) } { incr icol } {
	    rtf:startCell
	    rtf:ApplyPhrases
	    if { [info exists TableDef($irow,$icol)] } {
		rtf:writehtml "" $TableDef($irow,$icol) 1
	    }
	    rtf:UnApplyPhrases
	    rtf:endCell
	    if { [info exists TableDef($irow,$icol,colspan)] } {
		incr ncols $TableDef($irow,$icol,colspan)
		if { $ncols == $TableDef(ncols) } { break }
	    }
	}
	rtf:endRow
    }
    rtf:endTable
    unset TableDef
}

proc rtf:writehtmltableFinalNoTable {} {
    global TableDef

    for { set irow 1 } { $irow <= $TableDef(nrows) } { incr irow } {
	for { set icol 1 } { $icol <= $TableDef(ncols) } { incr icol } {
	    if { [info exists TableDef($irow,$icol)] } {
		rtf:writehtml "" $TableDef($irow,$icol) 1
	    }
	}
    }
    unset TableDef
}

proc rtf:writehtmltable { text } {
    global TableDef

    if { ![info exists TableDef(irow)] } {
	set TableDef(irow) 0
	set TableDef(icol) 0
	set TableDef(ncols) 0
	set TableDef(nrows) 0
	set TableDef(0,0) ""
    }

    set idx 0
    while 1 {
	set idxs ""
	regexp -indices -start $idx {<[^>]*>} $text idxs
	if { $idxs == "" } {
	    set idxend end
	} else {
	    set idxend [expr [lindex $idxs 0]-1]
	}
	append TableDef($TableDef(irow),$TableDef(icol)) [string range $text $idx $idxend]
	if { $idxend == "end" } { break }

	set tag [string range $text [expr [lindex $idxs 0]+1] [expr [lindex $idxs 1]-1]]
	set idx [expr [lindex $idxs 1]+1]
	set list [rtf:parsetag $tag]
	switch [lindex $list 0] {
	    table {
		# tables cannot nest
		rtf:writehtmltableFinalNoTable
		array set TableDef ""
		rtf:writehtmltable [string range $text $idx  end]
		return
	    }
	    tr {
		incr TableDef(irow)
		if { $TableDef(irow) > $TableDef(nrows) } {
		    set TableDef(nrows) $TableDef(irow)
		}
		set TableDef(icol) 0
		set TableDef($TableDef(irow),$TableDef(icol)) ""
	    }
	    td {
		set colspan 1
		foreach "name val" [lrange $list 1 end] {
		    switch $name {
			colspan {
			    set colspan $val
			}
		    }
		}
		incr TableDef(icol) 1
		if { $TableDef(icol) > $TableDef(ncols) } {
		    set TableDef(ncols) $TableDef(icol)
		}
		set TableDef($TableDef(irow),$TableDef(icol)) ""
		set TableDef($TableDef(irow),$TableDef(icol),colspan) $colspan
	    }
	    /tr {
		# nothing
	    }
	    /td {
		# nothing
	    }
	    /table {
		rtf:writehtmltableFinal
		rtf:writehtml "" [string range $text $idx end]
		return
	    }
	    default {
		append TableDef($TableDef(irow),$TableDef(icol)) <$tag>
	    }
	}
    }

}



proc rtf:writehtml { tag text { force 0 } } {
    global StyleState ParaOn PhraseOn TableDef

    set mapHTML2RTF {
	&gt; > 
	&lt; < 
	&nbsp; " " 
	\n ""
	&aacute; 
	&eacute; 
	&iacute; 
	&oacute; 
	&uacute; 
	&ntilde; 
	&middot; 
    }

    if { $tag != "" } {
	set text "<$tag>$text</$tag>"
    }

    if { !$force && [info exists TableDef] } {
	rtf:writehtmltable $text
	return
    }

    if { ![info exists StyleState] } {
	set StyleState ""
	set ParaOn 0
	set PhraseOn 0
    }

    set listlevel 0
    set idx 0
    while 1 {
	set idxs ""
	regexp -indices -start $idx {<[^>]*>} $text idxs
	if { $idxs == "" } {
	    set idxend end
	} else {
	    set idxend [expr [lindex $idxs 0]-1]
	}
	set t [string map $mapHTML2RTF [string range $text $idx $idxend]]
	if { $t != "" } {
	    rtf:text $t
	}
	if { $idxend == "end" } { break }

	set tag [string range $text [expr [lindex $idxs 0]+1] [expr [lindex $idxs 1]-1]]
	set idx [expr [lindex $idxs 1]+1]
	set list [rtf:parsetag $tag]
	switch [lindex $list 0] {
	    ul {
		incr listlevel
	    }
	    /ul {
		incr listlevel -1
	    }
	    li {
		set listlevelR $listlevel
		if { $listlevelR < 1 } { set listlevelR 1 }
		if { $listlevelR > 3 } { set listlevelR 3 }
		rtf:startParaR list$listlevelR
		rtf:bullet
	    }
	    table {
		array set TableDef ""
		rtf:writehtmltable [string range $text $idx  end]
		return
	    }
	    sym {
		lappend StyleState "font symbol"
		rtf:ApplyPhrases
	    }
	    font {
		set color ""
		set face ""
		foreach "name val" [lrange $list 1 end] {
		    switch $name {
			color {
			    set color red
			}
			face {
			    set face symbol
			}
		    }
		}
		if { $color != "" } {
		    if { $face != "" } {
			lappend StyleState "font $color $face"
		    } else {
			lappend StyleState "font $color"
		    }
		} elseif { $face != "" } {
		    lappend StyleState "font $face"
		}
		rtf:ApplyPhrases
	    }
            /font - /sym {
		set i 0
		while 1 {
		    set style [lindex $StyleState end-$i]
		    if { $style == "" } { break }
		    if { [lindex $style 0] == "font" } {
			set StyleState [lreplace $StyleState end-$i end-$i]
			rtf:ApplyPhrases
			break
		    }
		    incr i -1
		}
	    }
	    img {
		foreach "name val" [lrange $list 1 end] {
		    switch $name {
			src {
			    set file $val
			}
			align {
			    # nothing by now
			}
		    }
		}
		if { $file != "" } {
		    rtf:startParaR center
		    rtf:writejpgimage $file
		    rtf:endParaR
		}
	    }
	    sup {
		rtf:startPhrase superscript
	    }
	    sub {
		rtf:startPhrase subscript
	    }
	    /sup - /sub {
		rtf:endPhrase
	    }
	    title {
		#rtf:startParaR heading1
	    }
	    /title {
		rtf:endParaR
	    }
	    strong {
		lappend StyleState strong
		rtf:ApplyPhrases
	    }
	    /strong {
		set i [lsearch $StyleState strong]
		if { $i >= 0 } {
		    set StyleState [lreplace $StyleState $i $i]
		    rtf:ApplyPhrases
		}
	    }
	    p {
		set align Justify
		foreach "name val" [lrange $list 1 end] {
		    switch -- $name {
			align {
			    switch -- $val {
				center {
				    set align Center
				}
			    }
			}
		    }
		}
		if { $align == "Justify" } {
		    rtf:startParaR normal
		} else {
		    rtf:startParaR center
		}
	    }
	    h1 { rtf:startParaR heading1 }
	    h2 { rtf:startParaR heading2 }
	    h3 { rtf:startParaR heading3 }
	    h4 { rtf:startParaR heading4 }
	    /p - /h1 - /h2 - /h3 - /h4 {
		rtf:endParaR
	    }
	}
    }
}

proc rtf:BeginDocument { filename header } {

    set ::rtf_fp [open $filename w]
    rtf:start
    rtf:write {\viewkind1}
    rtf:divert LeftHeader
    rtf:startPara headerleft
    rtf:text $header
    rtf:undivert
    rtf:divert RightHeader
    rtf:startPara header
    rtf:insertField { STYLEREF "Ttulo 3" \* MERGEFORMAT }
    rtf:undivert
    rtf:divert LeftFooter
    rtf:startPara center
    rtf:text -
    rtf:special PageNumber
    rtf:text -
    rtf:undivert
    rtf:divert RightFooter
    rtf:startPara center
    rtf:text -
    rtf:special PageNumber
    rtf:text -
    rtf:undivert
    rtf:startPara heading1
    rtf:text "ndice"
    rtf:startPara normal
    rtf:insertField { TOC \* MERGEFORMAT } "Presione F9 para actualizar"
    rtf:startPara normal
}
proc rtf:EndDocument {} {
    rtf:end
    close $::rtf_fp
}

proc rtf:HTML2RTF { htmlfile rtffile header { append 0 } } {

    if { !$append } {
	rtf:BeginDocument $rtffile $header
	set fin [open $htmlfile r]
    } else {
	set fin [open $htmlfile r]
	rtf:AppendToDocument $rtffile
    }
    
    set cwd [pwd]
    cd [file dirname $htmlfile]
    while { ![eof $fin] } {
	gets $fin aa
	rtf:writehtml "" $aa
    }
    close $fin
    rtf:EndDocument
    cd $cwd
}

proc rtf:HTML2CSV { htmlfile csvfile header { append 0 } } {

    if { !$append } {
	set ::rtf_fp [open $csvfile w]
	set fin [open $htmlfile r]
    } else {
	set fin [open $htmlfile r]
	set ::rtf_fp [open $csvfile r+]
	seek $::rtf_fp 0 end
    }
    
    rtf:writecsv [read $fin]

    close $fin
    close $::rtf_fp
}

proc rtf:AppendToDocument { filename } {
    
    set fout [open $filename r+]
    set lastlinewithclosebracketPos -1
    set lastlinewithclosebracket ""
    seek $fout -400 end
    while { ![eof $fout] } {
	set pos [tell $fout]
	gets $fout aa
	if { [string first \} $aa] != -1 } {
	    set lastlinewithclosebracketPos $pos
	    set lastlinewithclosebracket $aa
	}
    }
    if { $lastlinewithclosebracketPos == -1 } {
	error "error: not found closing bracket"
    }
    seek $fout $lastlinewithclosebracketPos

    set pos [string last \} $lastlinewithclosebracket]
    puts $fout [string range $lastlinewithclosebracket 0 [expr $pos-1]]
    set ::rtf_fp $fout
    return $fout
}

# borrowed from package cvs
proc rtf:csv_join {values {sepChar ,}} {
    set out ""
    set sep {}
    foreach val $values {
	if {[string match "*\[\"$sepChar\]*" $val]} {
	    append out $sep\"[string map [list \" \"\"] $val]\"
	} else {
	    append out $sep$val
	}
	set sep $sepChar
    }
    return $out
}

proc rtf:writecsv { html } {

    set mapHTML2RTF {
	&gt; > 
	&lt; < 
	&nbsp; " " 
	\n ""
	&aacute; 
	&eacute; 
	&iacute; 
	&oacute; 
	&uacute; 
	&ntilde; 
	&middot; 
    }

    set fulltext ""
    set list ""
    foreach "- tag text" [regexp -inline -all {<([^>]*)>([^<]*)} $html] {
	switch -- $tag {
	    p - h1 - h2 - h3 - h4 - table - /table - tr - li {
		if { $fulltext != "" } {
		    lappend list $fulltext
		    set fulltext ""
		}
		if { $list != "" } {
		    puts $::rtf_fp [rtf:csv_join $list ";"]
		    set list ""
		}
	    }
	    td {
		if { $fulltext != "" } {
		    lappend list $fulltext
		    set fulltext ""
		}
	    }
	}
	append fulltext [string map $mapHTML2RTF $text]

	
    }
    if { $fulltext != "" } {
	lappend list $fulltext
    }
    if { $list != "" } {
	puts $::rtf_fp [rtf:csv_join $list ";"]
    }
}

# RAMSAN definitions

# Define stylesheet:
rtf:paraStyle normal "Normal" {
    Font roman
    FontSize 12pt
    LeftIndent 0pt
    Quadding Justify
}

rtf:paraStyle list1 "Lista 1" -basedon normal {
    LeftIndent 12pt
}

rtf:paraStyle list2 "Lista 2" -basedon normal {
    LeftIndent 24pt
}

rtf:paraStyle list3 "Lista 3" -basedon normal {
    LeftIndent 36pt
}

rtf:paraStyle image "Texto independiente" -basedon normal {
    Font roman
    FontSize 12pt
    LeftIndent 0pt
    Quadding Center
}

rtf:paraStyle header "Header" -basedon normal {
    Font roman
    FontSize 10pt
    LeftIndent 0pt
    Quadding Right
    BottomBorder thick
}

rtf:paraStyle headerleft "Header Left" -basedon normal {
    Font roman
    FontSize 10pt
    LeftIndent 0pt
    Quadding Left
    BottomBorder thick
}

rtf:paraStyle heading1 "Ttulo 1" -basedon normal {
    Font sans
    FontSize 16pt
    Bold 1
    LeftIndent 0pt
    Quadding Left
    SpaceBefore 12pt
    SpaceAfter 3pt
    KeepWithNext 1
}

rtf:paraStyle heading2 "Ttulo 2" -basedon normal {
    Font sans
    FontSize 14pt
    Bold 1
    Italic 1
    LeftIndent 0pt
    Quadding Left
    SpaceBefore 12pt
    SpaceAfter 3pt
    KeepWithNext 1
}

rtf:paraStyle heading3 "Ttulo 3" -basedon normal {
    Font sans
    FontSize 13pt
    Bold 1
    LeftIndent 0pt
    Quadding Left
    SpaceBefore 12pt
    SpaceAfter 3pt
    KeepWithNext 1
}

rtf:paraStyle heading4 "Ttulo 4" -basedon normal {
    Font sans
    FontSize 12pt
    Bold 1
    LeftIndent 0pt
    Quadding Left
    SpaceBefore 12pt
    SpaceAfter 3pt
    KeepWithNext 1
}

rtf:paraStyle center "Centered" -basedon normal {
    Quadding Center
    KeepWithNext 1
}

rtf:colorStyle orange {
    Red 128
    Green 128
    Blue 128
}

rtf:charStyle red "Red Font" {
    FGColor red
}

rtf:charStyle bold "Bold" {
    Bold 1
}

rtf:charStyle redbold "Red Bold" {
    FGColor red
    Bold 1
}

rtf:charStyle symbol "Symbol Font" {
    Font symbol
    FontSize 10pt
}

rtf:charStyle superscript "Superscript" {
    Superscript 1
}

rtf:charStyle subscript "Subscript" {
    Subscript 1
}

rtf:documentFormat {
    PaperSize A4
    TwoSide 1
    MirrorMargins 1
    Hyphenate 1
}

# foreach i [info commands rtf:*] {
#     rename $i $i-base
#     proc $i { args } "puts \"$i \$args\"; eval $i-base \$args"
# }

#*EOF*
