
proc LoadCasesInit { dir } {
    global ProblemTypePriv

    for { set i 1 } { $i <= 9 } { incr i } {
	set ProblemTypePriv(loadcaseimg$i) [image create photo \
		-file [file join $dir images l$i.gif]]
    } 
    set ProblemTypePriv(loadcaseimgldots) [image create photo \
	    -file [file join $dir images ldots.gif]]
    set ProblemTypePriv(loadcaseimgcheck) [image create photo \
		-file [file join $dir images check.gif]]
    set ProblemTypePriv(loadcaseimgrename) [image create photo \
		-file [file join $dir images rename.gif]]
    set ProblemTypePriv(loadcaseimgerase-cross) [image create photo \
		-file [file join $dir images erase-cross.gif]]
    set ProblemTypePriv(loadcaseimgchecked-off) [image create photo \
		-file [file join $dir images checked-off.gif]]
    set ProblemTypePriv(loadcaseimgelu) [image create photo \
		-file [file join $dir images elu.gif]]
    set ProblemTypePriv(loadcaseimgchecked-on) [image create photo \
		-file [file join $dir images checked-on.gif]]

    set ProblemTypePriv(imagesdir) [file join $dir images]


    set ProblemTypePriv(condfuns) "DWAssignCond DWEntitiesCond DWDrawConds \
	    DWUnassignSome DWUnassignAllThis DWUnassignAllCND"
    
    auto_load DWAssignCond
    foreach i $ProblemTypePriv(condfuns) {
	rename $i $i-base
	proc $i { GDN args } "LoadCasesCondFuncs $i-base \$GDN \$args"
    }
    
    set therearechanges [lindex [.central.s info Project] 2]

    GetLoadCaseNames
    ChangeToLoadCase 1
    
    if { !$therearechanges } {
	.central.s process *****CLEARPROJECTCHANGES
	#after idle .central.s clearprojectchanges
    }
}

proc LoadCasesBackToIntervalOne {} {

    delayedop -cancel changefunc "LoadCasesBackToIntervalOne"

    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1
    
    .central.s process escape escape escape escape data intervals changeinterval \
	    1 escape
    
    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }
}

proc LoadCasesDraw { what } {
    global ProblemTypePriv

    switch $what {
	constraints {
	    set intervaltochange 1
	}
	loads {
	    set intervaltochange [expr $ProblemTypePriv(currentloadcase)+1]
	}
    }

    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s process escape escape escape escape

    .central.s disable graphics 1
    .central.s disable warnline 1

    .central.s process data intervals changeinterval \
	    $intervaltochange escape

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    .central.s process escape escape escape escape data conditions drawcond -DrawAll-

    delayedop changefunc "LoadCasesBackToIntervalOne"
}

proc LoadCasesCondFuncs { func GDN args } {
    global ProblemTypePriv

    upvar \#0 $GDN GidData

    set intervaltochange 1
    if { $GidData(BOOK) == "Loads" } {
	set intervaltochange [expr $ProblemTypePriv(currentloadcase)+1]
    }
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    .central.s process escape escape escape escape
    .central.s process data intervals changeinterval \
	    $intervaltochange escape

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    set execline "$func $GDN"
    foreach i [lindex $args 0] { 
	lappend execline $i
    }
    eval $execline

    # it is cancelled before just for strange cases
    delayedop -cancel changefunc "LoadCasesBackToIntervalOne"
    set fail [delayedop changefunc "LoadCasesBackToIntervalOne"]
    if { $fail == 1 } {
	LoadCasesBackToIntervalOne
    }
}

proc GetLoadCaseNames {} {
    global ProblemTypePriv

    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    foreach "current total" [.central.s info intvdata num] {}

    set ProblemTypePriv(loadcasenames) ""
    for { set i 2 } { $i <= $total } { incr i } {
	.central.s process escape escape escape escape 
	.central.s process data intervals changeinterval \
		$i escape
	lappend ProblemTypePriv(loadcasenames) [lindex [.central.s info intvdata] 2]
    }
    .central.s process escape escape escape escape data intervals changeinterval \
	    1 escape

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

}

proc AddHelpToLoadCaseIcon { text } {
    global ProblemTypePriv

    set var ""
    foreach i [split [bind $ProblemTypePriv(loadcasewin) <Enter>] \n] {
	if { ![string match DisplayGidHelpLauncher* $i] } {
	    append var $i\n
	}
    }
    bind $ProblemTypePriv(loadcasewin) <Enter> $var
    
    GidHelpT $ProblemTypePriv(loadcasewin) $text
}

proc NewLoadCase { { name "" } {parent .gid } } {
    global ProblemTypePriv

    if { $name == "" } {
	set text "Enter new load case name"
	while 1 {
	    set var [list rambshell_dialogCombo $parent.temp {New load case} \
		    $text question 1]
	    
	    set args ""
	    foreach i $ProblemTypePriv(loadcasenames) {
		regsub -all {_} $i { } iname
		lappend args $iname
	    }
	    lappend var $args
	    set name [eval $var]
	    if { $name == "--CANCEL--" } { return }
	    set name [string trim $name "\" \t"]
	    regsub -all "\[ \t]+" $name {_} name
	    if { ![regexp {[a-zA-Z0-9]+} $name] } {
		set text [concat "Name must contain a letter or a number. "\
			"Enter new load case name"]
	    }
	    if { [lsearch $ProblemTypePriv(loadcasenames) $name] != -1 } {
		set text "Name already exists. Enter new load case name"
	    } else { break }
	}
    }
    
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    .central.s process escape escape escape escape data intervals newinterval \
	    yes no
    .central.s process escape escape escape escape data IntervalData $name \
	    escape
    
    lappend ProblemTypePriv(loadcasenames) $name
    set len [llength $ProblemTypePriv(loadcasenames)]

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    ChangeToLoadCase $len

    if { [info exists ProblemTypePriv(loadcasestype)] } {
	# trick to update the combined load cases frame
	set ProblemTypePriv(loadcasestype) $ProblemTypePriv(loadcasestype)
    }
} 

proc DeleteLoadCase { { parent .gid } } {
    global ProblemTypePriv

    if { [llength $ProblemTypePriv(loadcasenames)] == 1 } {
	WarnWin "Load case cannot be deleted. There must be, at least, one loadcase"
	return
    }

    set im1 [expr $ProblemTypePriv(currentloadcase)-1]
    set name [lindex $ProblemTypePriv(loadcasenames) $im1]
    regsub -all {_} $name { } vname

    set retval [tk_dialogRAM $parent.tempwiniw {Confirm deletion} \
	    "Are you sure to delete current loadcase '$vname'?" \
	    questhead 0 OK Cancel]
    if { $retval == 1 } { return }
    
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    set DisableWindows [.central.s disable windows]
    
    .central.s disable graphics 1
    .central.s disable warnline 1
    .central.s disable windows 1

    set ProblemTypePriv(loadcasenames) [lreplace $ProblemTypePriv(loadcasenames) \
	    $im1 $im1]

    .central.s process escape escape escape escape
    .central.s process data intervals ChangeInterval \
	    [expr $ProblemTypePriv(currentloadcase)+1]
    .central.s process escape escape escape escape data intervals DeleteInterval \
	    yes escape
    .central.s process escape escape escape escape data intervals ChangeInterval 1

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }
    if { !$DisableWindows } {  .central.s disable windows 0 }

    ChangeToLoadCase $ProblemTypePriv(currentloadcase)

    if { [info exists ProblemTypePriv(loadcasestype)] } {
	# trick to update the combined load cases frame
	set ProblemTypePriv(loadcasestype) $ProblemTypePriv(loadcasestype)
    }
} 

proc RenameLoadCase { { parent .gid } } {
    global ProblemTypePriv

    set im1 [expr $ProblemTypePriv(currentloadcase)-1]
    set name [lindex $ProblemTypePriv(loadcasenames) $im1]
    regsub -all {_} $name { } vname

    set text "Enter new name for loadcase '$vname'"

    while 1 {
	set var [list rambshell_dialogCombo $parent.temp {rename load case} \
		$text question 1]

	set args ""
	foreach i $ProblemTypePriv(loadcasenames) {
	    regsub -all {_} $i { } name
	    lappend args $name
	}
	lappend var $args
	set retval [eval $var]
	if { $retval == "--CANCEL--" } { return }
	set retval [string trim $retval "\" \t"]
	regsub -all "\[ \t]+" $retval {_} retval
	if { ![regexp {[a-zA-Z0-9]+} $retval] } {
	    set text [concat "Name must contain a letter or a number. "\
		    "Enter new name for loadcase '$vname'"]
	}
	if { [lsearch $ProblemTypePriv(loadcasenames) $retval] != -1 } {
	    set text "Name already exists. Enter new name for loadcase '$vname'"
	} else { break }
    }
    
    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    .central.s process escape escape escape escape 
    .central.s process data intervals ChangeInterval \
	    [expr $ProblemTypePriv(currentloadcase)+1]
    .central.s process escape escape escape escape data IntervalData $retval \
	    escape
    .central.s process escape escape escape escape data intervals ChangeInterval 1
    
    set ProblemTypePriv(loadcasenames) [lreplace $ProblemTypePriv(loadcasenames) \
	    $im1 $im1 $retval]

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    ChangeToLoadCase $ProblemTypePriv(currentloadcase)

    if { [info exists ProblemTypePriv(loadcasestype)] } {
	# trick to update the combined load cases frame
	set ProblemTypePriv(loadcasestype) $ProblemTypePriv(loadcasestype)
    }
} 

proc ChangeToLoadCaseCombo {} {
    global ProblemTypePriv

    set name $ProblemTypePriv(loadcasecombotext)
    if { $name == "" } { return }
    regsub -all { } $name {_} name
    set num [lsearch $ProblemTypePriv(loadcasenames) $name]
    incr num
    ChangeToLoadCase $num
}

proc ChangeToLoadCase { num } {
    global ProblemTypePriv

    set len [llength $ProblemTypePriv(loadcasenames)]

    if { !$len } {
	NewLoadCase "Load_case_1"
	#this func is called by NewLoadCase
	return
    }

    if { $num > $len } { set num $len }

    if { [info exists  ProblemTypePriv(currentloadcase)] && \
	    $ProblemTypePriv(currentloadcase) != $num } {
	.central.s process escape escape escape escape
    }

    if { [info exists ProblemTypePriv(loadcasewin)] && \
	    [winfo exists $ProblemTypePriv(loadcasewin)] } {
	
	if { $num < 10 } {
	    $ProblemTypePriv(loadcasewin) conf -image $ProblemTypePriv(loadcaseimg$num)
	} else {
	    $ProblemTypePriv(loadcasewin) conf -image $ProblemTypePriv(loadcaseimgldots)
	}
	
	set menu [$ProblemTypePriv(loadcasewin) cget -menu]
	$menu del 0 end
	for { set i 1 } { $i <= $len } { incr i } {
	    set im1 [expr $i-1]
	    set name [lindex $ProblemTypePriv(loadcasenames) $im1]
	    regsub -all {_} $name { } name
	    set name "Loadcase: \[$i] $name"
	    if { $i != $num } {
		$menu add command
	    } else {
		$menu add checkbutton -onvalue 0
		AddHelpToLoadCaseIcon $name
	    }
	    $menu entryconf $im1 -label $name -command \
		    "ChangeToLoadCase $i"
	}
	$menu add separator
	$menu add command -label New -command NewLoadCase
	$menu add command -label Delete -command DeleteLoadCase
	$menu add command -label Rename -command RenameLoadCase
	$menu add command -label "Load case window..." -command LoadCaseWindow
    }
    if { [info exists ProblemTypePriv(loadcasecombo)] && \
	    [winfo exists $ProblemTypePriv(loadcasecombo)] } {
	$ProblemTypePriv(loadcasecombo) conf -editable 1
	set lcns ""
	set inum 1
	foreach i $ProblemTypePriv(loadcasenames) {
	    regsub -all {_} $i { } name
	    lappend lcns $name
	    if { $inum == $num } {
		set cname $name
	    }
	    incr inum
	}
	$ProblemTypePriv(loadcasecombo) conf -history $lcns
	$ProblemTypePriv(loadcasecombo) del 0 end
	$ProblemTypePriv(loadcasecombo) ins end $cname
	$ProblemTypePriv(loadcasecombo) conf -editable 0
    }

    set ProblemTypePriv(currentloadcase) $num
}

proc LoadCaseWindow {} {
    global ProblemTypePriv

    set FontHeight [font metric NormalFont -linespace]
    set FramesSeparation [expr $FontHeight/2+1]
    set w .gid.loadcases

    if { [info procs InitWindow] != "" } {
	InitWindow $w "Load cases" PreLoadCasesWindowGeom LoadCaseWindow
    } else {
	toplevel $w
    }

    framelabel $w.fl "load cases type"
    frame $w.fl.f
    grid $w.fl.f -pady $FramesSeparation -sticky nw

    radiobutton $w.fl.f.r1 -text "Do not consider loadcases" \
	    -var ProblemTypePriv(loadcasestype) -value none
    radiobutton $w.fl.f.r2 -text "One result for every loadcase" \
	    -var ProblemTypePriv(loadcasestype) -value independent
    radiobutton $w.fl.f.r3 -text "Use combined loadcases" \
	    -var ProblemTypePriv(loadcasestype) -value combined
    
    GidHelp "$w.fl.f.r1 $w.fl.f.r2 $w.fl.f.r3" "These options control the way of dealing with"\
	    " loadcases:\n.-Do not consider loadcases: There will be one combined loadcase that"\
	    " will contain all simple loadcases with factor 1.0\n.-One results for every laodcase:"\
	    " There will be one combined laodcase for every simple loadcase. User can enter the"\
	    " amplification factors.\n.-Use combined loadcases: It is possible to define as many"\
	    " combined loadcases as desired. Every one will be composed by a combination of simple"\
	    " loadcases with amplification factors entered by the user."

    grid $w.fl.f.r1 -sticky nw
    grid $w.fl.f.r2 -sticky nw
    grid $w.fl.f.r3 -sticky nw

    framelabel $w.f1b "Current load case"
    frame $w.f1b.f
    grid $w.f1b.f -pady $FramesSeparation -sticky nwes -padx 2 -row 1 -column 1
    grid columnconf $w.f1b 1 -weight 1
    set lcns ""
    set inum 1
    foreach i $ProblemTypePriv(loadcasenames) {
	regsub -all {_} $i { } name
	lappend lcns $name
	if { $inum == $ProblemTypePriv(currentloadcase) } {
	    set cname $name
	}
	incr inum
    }
    set ProblemTypePriv(loadcasecombo) [combobox $w.f1b.f.c -editable 1 \
	    -keephistory 0 -history $lcns -textvar ProblemTypePriv(loadcasecombotext)]
    $w.f1b.f.c del 0 end
    $w.f1b.f.c ins end $cname
    $w.f1b.f.c conf -editable 0

    GidHelp $w.f1b.f.c "This is a list of all created simple loadcases. In this menu it is "\
	    "selected the active loadcase. All new loads assigned to entities will be inserted"\
	    " in the active loadcase. The active loadcase can also be changed in the Ram Series"\
	    " toolbars buttons."

    trace var ProblemTypePriv(loadcasecombotext) w "ChangeToLoadCaseCombo ;#"

    button $w.f1b.f.b1 -image $ProblemTypePriv(loadcaseimgcheck) -padx 0 -bd 1 -pady 0 \
	    -command "NewLoadCase {} $w"
    GidHelp $w.f1b.f.b1 "Creates new simple load case"
    button $w.f1b.f.b2 -image $ProblemTypePriv(loadcaseimgrename) -padx 0 -bd 1 -pady 0 \
	    -command "RenameLoadCase $w"
    GidHelp $w.f1b.f.b2 "Renames current simple load case"
    button $w.f1b.f.b3 -image $ProblemTypePriv(loadcaseimgerase-cross) -padx 0 -bd 1 -pady 0 \
	    -command "DeleteLoadCase $w"
    GidHelp $w.f1b.f.b3 "Deletes current simple load case.\nAll loads contained in this loadcase"\
	    " will be deleted, too."

    grid $w.f1b.f.c -row 1 -column 1 -padx 2 -sticky ew
    grid $w.f1b.f.b1 -row 1 -column 2 -padx 0
    grid $w.f1b.f.b2 -row 1 -column 3 -padx 0
    grid $w.f1b.f.b3 -row 1 -column 4 -padx 0
    grid columnconf $w.f1b.f 1 -weight 1

    framelabel $w.f2 "combined load cases"
    frame $w.f2.f
    grid $w.f2.f -pady $FramesSeparation -sticky nwes -row 1 -column 1
    grid columnconf $w.f2 1 -weight 1
    grid rowconf $w.f2 1 -weight 1
    canvas $w.f2.f.c -xscrollcommand "$w.f2.f.sx set" -yscrollcommand \
	    "$w.f2.f.sy set" -highlightthickness 0
    frame $w.f2.f.c.f
    $w.f2.f.c create window 0 0 -window $w.f2.f.c.f -anchor nw
    scrollbar $w.f2.f.sx -orient h -command "$w.f2.f.c xview"
    scrollbar $w.f2.f.sy -orient v -command "$w.f2.f.c yview"

    proc ScrollHideShow  { c sx sy } {
	foreach "a b" [$c xview] ""
	if { $a == 0.0 && $b == 1.0 } {
	    grid remove $sx
	} else {
	    grid $sx
	}
	foreach "a b" [$c yview] ""
	if { $a == 0.0 && $b == 1.0 } {
	    grid remove $sy
	} else {
	    grid $sy
	}
    }
    bind $w.f2.f.c <Configure> "after idle ScrollHideShow $w.f2.f.c $w.f2.f.sx $w.f2.f.sy"


    grid $w.f2.f.c -sticky nsew -row 1 -column 1
    grid $w.f2.f.sy -sticky ns -row 1 -column 2
    grid $w.f2.f.sx -sticky ew -row 2 -column 1
    grid columnconf $w.f2.f 1 -weight 1
    grid rowconf $w.f2.f 1 -weight 1
    
    frame $w.buts
    button $w.buts.acc -text "Accept data" -und 0 -width 11 -command \
	    "AcceptLCData $w.f2.f.c.f $w.f2.f.c"
    button $w.buts.cancel -text "Close" -und 0 -width 11 -command \
	    "destroy $w"
    button $w.buts.hp -image [imagefromram questionarrow.gif] -command "PickHelp $w.buts.hp" \
	    -highlightthickness 0

    GidHelp $w.buts.acc "This button stores all the changes made inside this window."
    GidHelp $w.buts.cancel "This button closes the window. If changes are to be accepted, press"\
	    " before button 'Accept data'."

    bind $w <Alt-a> "tkButtonInvoke $w.buts.acc"
    bind $w <Alt-c> "tkButtonInvoke $w.buts.cancel"
    bind $w <Escape> "tkButtonInvoke $w.buts.cancel"
    grid $w.buts.acc -row 1 -column 1 -padx 2
    grid $w.buts.cancel -row 1 -column 2 -padx 2
    grid $w.buts.hp -row 1 -column 3 -padx 2

    grid $w.fl -pady $FramesSeparation -sticky nsew -row 1 -column 1 -padx 1
    grid $w.f1b -pady $FramesSeparation -sticky nsew -row 1 -column 2 -padx 1
    grid $w.f2 -pady $FramesSeparation -sticky nwes -row 2 -column 1 \
	    -columnspan 2 -padx 1
    grid $w.buts -sticky ew -row 3 -column 1 -columnspan 2 -padx 1 -pady 2
    grid columnconf $w 1 -weight 1
    grid columnconf $w 2 -weight 1
    grid rowconf $w 2 -weight 1

    trace var ProblemTypePriv(loadcasestype) w "CombinedLCUpdate $w.f2.f.c.f $w.f2.f.c ;#"

    set gendata [.central.s info gendata]
    foreach "name value" [lrange $gendata 1 end] {
	if { [string match Activate_Load_Cases* $name] } {
	    set lct $value
	} elseif { [string match Combined_load_cases* $name] } {
	    set ProblemTypePriv(combined) [lrange $value 2 end]
	}
    }
    set ProblemTypePriv(loadcasestype) $lct

}

proc ChangeStateChildren { w newstate } {
    foreach i [winfo children $w] {
	catch { $i conf -state $newstate }
	if { $newstate == "disabled" } {
	    catch {
		$i conf -fg grey60
	    }
	    bind $i <Double-1> ""
	    bind $i <3> ""
	} else {
	    catch {
		$i conf -fg black
	    }
	}
	ChangeStateChildren $i $newstate
    }
}

proc AcceptLCData { w canvas } {
    global ProblemTypePriv

    ComputeValuesFromCanvas $w

    set DisableGraphics [.central.s disable graphics]
    set DisableWarnLine [.central.s disable warnline]
    
    .central.s disable graphics 1
    .central.s disable warnline 1

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }

    set newgendata ""
    foreach "name value" [lrange [.central.s info gendata] 1 end] {
	if { [string match Activate_Load_Cases* $name] } {
	    lappend newgendata $ProblemTypePriv(loadcasestype)
	} elseif { [string match Combined_load_cases* $name] } {
# 	    set numlost 0
# 	    while { [string length $ProblemTypePriv(combined)] > 1000 } {
# 		set len [llength $ProblemTypePriv(combined)]
# 		set ProblemTypePriv(combined) [lrange $ProblemTypePriv(combined) 0 \
# 			[expr $len-4]]
# 		incr numlost
# 	    }
# 	    if { $numlost } {
# 		WarnWin [concat "Error: One internal limit has been reached. $numlost "\
# 			"combined load cases could not be maintained"]
# 		CombinedLCUpdate $w $canvas 0
# 	    }

	    lappend newgendata [concat "#N# [llength $ProblemTypePriv(combined)] "\
		    "$ProblemTypePriv(combined)"]
	} else {
	    lappend newgendata $value
	}
    }
    set var ".central.s process escape escape escape escape data ProblemData "
    append var "$newgendata escape"
    eval $var

    if { !$DisableGraphics } { .central.s disable graphics 0 }
    if { !$DisableWarnLine } { .central.s disable warnline 0 }
}

proc ComputeValuesFromCanvas { w } {
    global ProblemTypePriv

    set ProblemTypePriv(combined) ""
    set i 1
    while 1 {
	if { ![winfo exists $w.v$i] } { break }
	set name [$w.v$i cget -text]
	regsub -all { } $name {_} name
	set values ""
	set j 1
	while 1 {
	    if { ![winfo exists $w.v$i-$j] } { break }
	    if { [$w.v$i-$j cget -state] == "normal" } {
		set lname [$w.l$j cget -text]
		regsub -all { } $lname {_} lname
		set value [$w.v$i-$j get]
		
		if { [catch { set value [expr double($value)] }] } {
		    set value 0.0
		}
		if { $value != 0.0 } {
		    if { $values != "" } { append values , }
		    append values $value,$lname
		}

	    }
	    incr j
	}
	if { $values != "" } {
	    lappend ProblemTypePriv(combined) $name
	    lappend ProblemTypePriv(combined) $ProblemTypePriv(isstrength,$i)
	    lappend ProblemTypePriv(combined) $values
	}
	incr i
    }
}

proc LCNameExists { newname } {
    global ProblemTypePriv
    
    regsub -all { } $newname {_} newname
    foreach "name isstrength values" $ProblemTypePriv(combined) {
	if { $newname == $name } { return 1 }
    }
    return 0
}

proc LCHelpForCombined { w x y } {
    set pos [lsearch [bind $w <ButtonPress-3>] DisplayGidHelpMenu]
    if { $pos != -1 } {
	incr pos 2
	set text [lindex [bind $w <ButtonPress-3>] $pos]
    } else {
	set text "There is no help for this item"
    }
    DisplayGidHelp $w $text menu $x $y
}

proc CreateLCMenu {w wl canvas inum x y } {

    catch { destroy $w.menu }
    menu $w.menu
    $w.menu add command -label "Insert after" -command \
	    "CombinedLCOperations $w $canvas [expr $inum+1] insertrow"
    $w.menu add separator
    $w.menu add command -label "Insert before" -command \
	    "CombinedLCOperations $w $canvas $inum insertrow"
    $w.menu add command -label "Delete" -command \
	    "CombinedLCOperations $w $canvas $inum deleterow"
    $w.menu add separator
    $w.menu add command -label "Change factor" -command \
	    "CombinedLCOperations $w $canvas $inum setvalue"
    $w.menu add command -label "Rename" -command \
	    "CombinedLCOperations $w $canvas $inum rename"
    $w.menu add separator
    $w.menu add command -label "Help" -command \
	    "LCHelpForCombined $wl $x $y"
    tk_popup $w.menu $x $y 0
}

proc CombinedLCOperations { w canvas inum what { x 0 } } {
    global ProblemTypePriv

    switch $what {
	rename {
	    entry $w.v$inum.e -relief [$w.v$inum cget -relief] -bd [$w.v$inum cget -bd] \
		    -fg [$w.v$inum cget -fg] -bg [$w.v$inum cget -bg] \
		    -highlightthickness 0
	    $w.v$inum.e del 0 end
	    $w.v$inum.e ins end [$w.v$inum cget -text]
	    place $w.v$inum.e -in $w.v$inum -x 0 -y 0 -anchor nw -relwidth 1 -relheight 1 \
		    -bordermode outside

	    focus $w.v$inum.e
	    tkwait visibility $w.v$inum.e
	    grab $w.v$inum.e
	    $w.v$inum.e icursor @$x

	    ComputeValuesFromCanvas $w
	    bind $w.v$inum.e <Return> {
		set newname [%W get]
		if { ![LCNameExists $newname] } {
		    [winfo parent %W] conf -text $newname
		}
		destroy %W
		break
	    }
	    bind $w.v$inum.e <Escape> "destroy %W ; break"
	    bind $w.v$inum.e <ButtonPress> {
		if { %x < 0 || %x > [winfo width %W] || %y < 0 || %y > [winfo height %W] } {
		    set newname [%W get]
		    if { ![LCNameExists $newname] } {
			[winfo parent %W] conf -text $newname
		    }
		    destroy %W
		    break
		}
	    }
	}
	setvalue {
	    set name [$w.v$inum cget -text]
	    set retval [tk_dialogEntryRAM $w.temp "Enter factor" \
		    "Enter a factor for all the load cases of '$name'" \
		    question real 1.0]
	    if { $retval == "--CANCEL--" } { return }
	    ComputeValuesFromCanvas $w
	    regsub -all { } $name {_} name

	    set values ""
	    set first 1
	    foreach i $ProblemTypePriv(loadcasenames) {
		if { !$first } { 
		    append values ,
		} else { set first 0 }
		append values $retval,$i
	    }
	    set ProblemTypePriv(combined) [lreplace $ProblemTypePriv(combined) \
		    [expr ($inum-1)*3+2] [expr ($inum-1)*3+2] $values]
	    CombinedLCUpdate $w $canvas 0
	}
	insertrow {
	    if {$inum == "end" } {
		set inum [expr [llength $ProblemTypePriv(combined)]/3+1]
	    }

	    ComputeValuesFromCanvas $w
	    set i 1
	    while 1 {
		set newname Combined_$i
		if { ![LCNameExists $newname] } { break }
		incr i
	    }
	    set values ""
	    set first 1
	    foreach i $ProblemTypePriv(loadcasenames) {
		if { !$first } { 
		    append values ,
		} else { set first 0 }
		append values 1.0,$i
	    }
	    set ProblemTypePriv(combined) [linsert $ProblemTypePriv(combined) \
		    [expr ($inum-1)*3] $newname 1 $values]
	    CombinedLCUpdate $w $canvas 0
	}
	deleterow {
	    if {$inum == "end" } {
		set inum [expr [llength $ProblemTypePriv(combined)]/3]
	    }

	    ComputeValuesFromCanvas $w
	    set ProblemTypePriv(combined) [lreplace $ProblemTypePriv(combined) \
		    [expr ($inum-1)*3] [expr ($inum-1)*3+2]]
	    CombinedLCUpdate $w $canvas 0
	}
    }
}

proc CombinedLCUpdate { w canvas { recompute 1 } } {
    global ProblemTypePriv

    if { ![winfo exists $w] } { return }
    if { [winfo children $w] != "" && $recompute } { ComputeValuesFromCanvas $w }

    foreach i [winfo children $w] { destroy $i }
        
    set ncases [llength $ProblemTypePriv(loadcasenames)]

    label $w.l0 -image $ProblemTypePriv(loadcaseimgelu) -bd 1 -relief raised
    GidHelp $w.l0 "When column 'ELU' is active means that 'combined "\
	    "load cases' are used to calculate strengths. If not, they are preferred "\
	    "to calculate displacements. This option do not change the analysis. It is only"\
	    " used to dimension concrete."
    grid $w.l0 -row 0 -column 1 -sticky ew
    for { set i 1 } { $i <= $ncases } { incr i } {
	set name [lindex $ProblemTypePriv(loadcasenames) \
		[expr $i-1]]
	regsub -all {_} $name { } name
	label $w.l$i -text $name -bd 1 -relief raised
	GidHelp $w.l$i "These are the simple loadcases.\nA combination of simple loadcases with its"\
		" amplification factors defines a combined loadcase. There will be one result"\
		" for every combined loadcase."
	grid $w.l$i -row 0 -column [expr $i+1]  -sticky ew
    }

    if { [info exists ProblemTypePriv(loadcasestypeold)] && $ProblemTypePriv(loadcasestypeold) != \
	    $ProblemTypePriv(loadcasestype) } {
	set ProblemTypePriv(combined) ""
    }

    if { $ProblemTypePriv(combined) == "" } {
	set ProblemTypePriv(combined) "Combined_1 1 "
	set first 1
	foreach i $ProblemTypePriv(loadcasenames) {
	    if { $first } {
		set first 0
	    } else { append ProblemTypePriv(combined) , }
	    append ProblemTypePriv(combined) "1.0,$i"
	}
    }
    if { $ProblemTypePriv(loadcasestype) == "independent" } {
	set combined_base $ProblemTypePriv(combined)
	set ProblemTypePriv(combined) ""
	for { set i 1 } { $i <= $ncases } { incr i } {
	    set name [lindex $ProblemTypePriv(loadcasenames) \
		    [expr $i-1]]
	    set ipos [expr ($i-1)*3]
	    lappend ProblemTypePriv(combined) $name
	    if { [lindex $combined_base $ipos] == "" } {
		lappend ProblemTypePriv(combined) 1
		lappend ProblemTypePriv(combined) 1.0,$name
	    } else {
		lappend ProblemTypePriv(combined) [lindex $combined_base [expr $ipos+1]]
		set list [split [lindex $combined_base [expr $ipos+2]] ,]
		set found 0
		foreach "factor lc" $list {
		    if { $name == $lc } {
			set ffactor $factor
			set found 1
			break
		    } 
		}
		if { !$found } { set ffactor 1.0 }
		lappend ProblemTypePriv(combined) $ffactor,$name
	    }
	}
    }
    
    set i 1
    foreach "name isstrength values" $ProblemTypePriv(combined) {
	regsub -all {_} $name { } name
	label $w.v$i -text $name -bd 1 -relief raised -anchor e


	bind $w.v$i <Double-1> "CombinedLCOperations $w $canvas $i rename %x"

	GidHelp $w.v$i "These are the combined loadcases. To change the name, double-click on"\
		"them. To insert a new one, press right-button-mouse over them.\nA combination "\
		"of simple loadcases with its"\
		" amplification factors defines a combined loadcase. There will be one result"\
		" for every combined loadcase."

	# trick to enter the help but make the right button menu work
	set comm [bind $w.v$i <ButtonPress-3>]
	bind $w.v$i <ButtonPress-3> "CreateLCMenu $w $w.v$i $canvas $i %X %Y ; break ; $comm"

	grid $w.v$i -row $i -column 0 -sticky ew -padx 2
	checkbutton $w.c$i -var ProblemTypePriv(isstrength,$i) \
		-image $ProblemTypePriv(loadcaseimgchecked-off) \
		-selectimage $ProblemTypePriv(loadcaseimgchecked-on) \
		-indicatoron 0 -bd 0 -selectcolor ""   
	GidHelp $w.c$i "When column 'ELU' is active means that 'combined "\
		"load cases' are used to calculate strengths. If not, they are preferred "\
		"to calculate displacements. This option do not change the analysis. It is only"\
		" used to dimension concrete."

	set ProblemTypePriv(isstrength,$i) $isstrength
	grid $w.c$i -row $i -column 1

	set list [split $values ,]
	for { set j 1 } { $j <= $ncases } { incr j } {
	    set lname [lindex $ProblemTypePriv(loadcasenames) \
		    [expr $j-1]]
	    entry $w.v$i-$j -width 3 -bd 1

	    GidHelp $w.v$i-$j "It is possible to enter here the amplification factors for every"\
		    "simple loadcase related to every combined loadcase. A factor of 0.0 means not"\
		    "to consider that simple loadcase inside the combined one\n A combination "\
		    "of simple loadcases with its"\
		    " amplification factors defines a combined loadcase. There will be one result"\
		    " for every combined loadcase."

	    grid $w.v$i-$j -row $i -column [expr $j+1] -sticky ew
	    
	    set found 0
	    if { $list == "allone" } {
		set ffactor 1.0
		set found 1
	    } else {
		foreach "factor lc" $list {
		    if { $lname == $lc } {
			set ffactor $factor
			set found 1
			break
		    } 
		}
	    }
	    if { !$found } { set ffactor 0.0 }
	    $w.v$i-$j ins end [format %g $ffactor]
	    if { $ProblemTypePriv(loadcasestype) == "independent" && \
		    $i != $j } {
		$w.v$i-$j conf -state disabled -fg grey60
	    }
	}
	incr i
    }

    menubutton $w.arrow -image [imagefrom arrow_down_vsmall.gif] -menu $w.arrow.menu -relief raised
    menu $w.arrow.menu
    $w.arrow.menu add command -label "Insert" -command \
	    "CombinedLCOperations $w $canvas end insertrow"
    $w.arrow.menu add command -label "Delete" -command \
	    "CombinedLCOperations $w $canvas end deleterow"
    $w.arrow.menu add separator
    $w.arrow.menu add command -label "Help" -command \
	    "LCHelpForCombined $w.arrow \[winfo rootx $w.arrow.menu] \[winfo rooty $w.arrow.menu]"


    GidHelp $w.arrow "These are the combined loadcases. To change the name, double-click on"\
	    "them. To insert a new one, press right-button-mouse over them.\nA combination "\
	    "of simple loadcases with its"\
	    " amplification factors defines a combined loadcase. There will be one result"\
	    " for every combined loadcase."
    
    grid $w.arrow -row $i -column 0 -sticky w -padx 2

    if { $ProblemTypePriv(loadcasestype) == "none" } {
	ChangeStateChildren $w disabled
    }

    update idletasks
    $canvas conf -scrollregion "0 0 [winfo reqwidth $w] \
	    [winfo reqheight $w]"
    event generate $canvas <Configure>
    set ProblemTypePriv(loadcasestypeold) $ProblemTypePriv(loadcasestype)

}


proc framelabel { w text } {
    frame $w -bd 2 -relief groove
    set i 0
    set parent [winfo parent $w]
    if { $parent == "." } { set parent "" }
    set label $parent.__label$i
    while { [winfo exists $label] } {
	incr i
	set label $parent.__label$i
    }
    label $label -text $text -pady 0
    place $label -in $w -x 10 -y -1 -anchor w -bordermode outside
    after idle raise $label
    return $w
}

#
# This procedure displays a dialog box, with a combobox, waits for a button in 
# the dialog to be invoked, then returns the entry.
# If cancel is pressed, it returns: --CANCEL--
#
# Arguments:
# w -		Window to use for dialog top-level.
# title -	Title to display in dialog's decorative frame.
# text -	Message to display in dialog.
# bitmap -	Bitmap to display in dialog (empty string means none).
# AcceptNew     Can be: TCL_BOOLEAN or 2 (then, word must begin with a letter)
#                       3 (then, one positive number must be entered)
#
# args some words to add to the combo
#

proc rambshell_dialogCombo {w title text bitmap AcceptNew args} {
    global tkPriv GidPriv tcl_platform

    # 1. Create the top-level window and divide it into top, middle
    # and bottom parts.

    while { [winfo exists $w] } { append w a }
    catch {destroy $w}
    toplevel $w -class OverlayWindow

    wm title $w $title
    wm iconname $w Dialog
    wm protocol $w WM_DELETE_WINDOW { }

    if { $tcl_platform(platform) == "windows" } {
	wm transient $w [winfo toplevel [winfo parent $w]]
    }

    frame $w.bot -relief raised -bd 1
    pack $w.bot -side bottom -fill both
    frame $w.top -relief raised -bd 1
    pack $w.top -side top -fill both -expand 1

    frame $w.middle -relief raised -bd 1

    set GidPriv(entry) ""
    set iseditable $AcceptNew
    if { $iseditable > 1 } { set  iseditable 1 }
    combobox $w.middle.e -editable $iseditable \
	    -textvariable GidPriv(entry) -bd 1

    set last ""
    foreach i [lindex $args 0] {
	if { $i == $last } { continue }
	$w.middle.e add $i
	set last $i
    }

    if { $AcceptNew == 0 || $AcceptNew == 3 } {
	set GidPriv(entry) [lindex [lindex $args 0] 0]
	$w.middle.e sel from 0
	$w.middle.e sel to end
    }

    pack $w.middle.e -side top -padx 5 -fill x -expand 1
    pack $w.middle -fill x -expand 1


    # 2. Fill the top part with bitmap and message (use the option
    # database for -wraplength so that it can be overridden by
    # the caller).

    option add *Dialog.msg.wrapLength 3i widgetDefault
    label $w.msg -justify left -text $text -wraplength 75m -font BigFont

    pack $w.msg -in $w.top -side right -expand 1 -fill both -padx 3m -pady 3m
    if {$bitmap != ""} {
	label $w.bitmap -bitmap $bitmap
	pack $w.bitmap -in $w.top -side left -padx 3m -pady 3m
    }

    bind $w <Return> "
       $w.button0 configure -state active -relief sunken
       update idletasks
       after 100
       set tkPriv(button) 0
    "

    # 3. Create a row of buttons at the bottom of the dialog.

    set UnderLineList ""
    set i 0

    set default 0
    set args "OK Cancel"
    set lentext 0
    foreach but $args {
	if { [string length $but] > $lentext } {
	    set lentext [string length $but]
	}
    }
    foreach but $args {
	button $w.button$i -text $but -command "set tkPriv(button) $i" \
		-width $lentext -bd 1
	pack $w.button$i -in $w.bot -side left -expand 1 \
		-padx 3m -pady 2m
	bind $w.button$i <Return> "
	    $w.button$i configure -state active -relief sunken
	    update idletasks
	    after 100
	    set tkPriv(button) $i
            break
	"
	if { [regexp -nocase cancel $but] } "
	    bind $w <Escape> \"
	    $w.button$i configure -state active -relief sunken
	    update idletasks
	    after 100
	    set tkPriv(button) $i
	\""
	incr i
    }

    # 5. Withdraw the window, then update all the geometry information
    # so we know how big it wants to be, then center the window in the
    # display and de-iconify it.

    if { [winfo parent $w] == ".gid" } {
	bind $w <Destroy> [list DestroyWinInDialogs $w GeomDialog]
    }
    if { [info exists GidPriv(GeomDialog)] && [winfo parent $w] == ".gid" } {
	WmGidGeom $w $GidPriv(GeomDialog)
    } else {
	wm withdraw $w
	update idletasks
	# extrange errors with updates
	if { ![winfo exists $w] } { return 0}
	
	set pare [winfo toplevel [winfo parent $w]]
	set x [expr [winfo x $pare]+[winfo width $pare ]/2- [winfo reqwidth $w]/2]
	set y [expr [winfo y $pare]+[winfo height $pare ]/2- [winfo reqheight $w]]
	if { $x < 0 } { set x 0 }
	if { $y < 0 } { set y 0 }
	WmGidGeom $w +$x+$y
	update
	wm deiconify $w
    }

    # 6. Set a grab and claim the focus too.

    set oldFocus [focus]
    tkwait visibility $w
    set oldGrab [grab current $w]
    if {$oldGrab != ""} {
	set grabStatus [grab status $oldGrab]
    }
    grab $w

    focus [$w.middle.e cget entry]
    #     after 100 catch [list "focus -force $w.middle.e"]

 
    # 7. Wait for the user to respond, then restore the focus and
    # return the index of the selected button.  Restore the focus
    # before deleting the window, since otherwise the window manager
    # may take the focus away so we can't redirect it.  Finally,
    # restore any grab that was in effect.

    while 1 {
	tkwait variable tkPriv(button)
	if { $tkPriv(button) == 1} { break }
	set result [$w.middle.e get]
	
	if { $AcceptNew == 2 && ![regexp {^[a-zA-Z]} $result] } {
	    set errorRes "Word must begin with a letter"
	} elseif { $AcceptNew == 3 && ![regexp \
		{^[  ]*([+]?[0-9]*\.?[0-9]*([eE][+-]?[0-9]+)?)[  ]*$} \
		$result] } {
	    set errorRes "One positive number must be entered"
	}	
	if { [info exists errorRes] } {
	    tk_dialogRAM $w.tempwin {Error window} $errorRes error 0 OK
	    unset errorRes
	} else { break }
    }
    
    catch {focus $oldFocus}
    destroy $w
    if {$oldGrab != ""} {
	if {$grabStatus == "global"} {
	    grab -global $oldGrab
	} else {
	    grab $oldGrab
	}
    }
    if { $tkPriv(button) == 1} { return --CANCEL-- }

    if { [llength $GidPriv(entry)] > 1 } { 
	return "\"$GidPriv(entry)\"" 
    } else {
	return $GidPriv(entry)
    }
}

#  proc WarnWin { text } {
#      tk_dialog .temp {New load case} \
#  	    $text question 0 OK
#  }
#  #source "/gid project/scripts/combo.tcl"
#  source /home/ramsan/bin/scripts/combo.tcl
#  font create NormalFont -family "MS Sans Serif" -size 10
#  option add *font NormalFont
#  wm withdraw .
#  after idle bind .loadcases <Destroy> exit

#  set ProblemTypePriv(loadcasenames) {"peso_propio" "sobrecarga_variable" \
#  	"viento"}
#  set dir .
#  set ProblemTypePriv(currentloadcase) 1

#  foreach i [glob [file join $dir images *.gif]] {
#      set ProblemTypePriv(loadcaseimg[file root [file tail $i]]) [image create photo \
#  	    -file $i]
#  }

#  proc GidHelp { args } {}
#  LoadCaseWindow


proc AddReactions { fr text } {
    global GidPriv

    if { [trace vinfo GidPriv(selection)] != "" } {  trace var GidPriv(selection) w "" }
    frame $fr.__f -bd 2 -relief ridge -width [expr [winfo width $fr]-0] \
	    -height [expr [winfo height $fr]-0]
    label $fr.__f.l -text "Select nodes"
    button $fr.__f.b -text Finish -command "EndAddReactions $fr.__f $text 1"
    pack $fr.__f.l $fr.__f.b -side left -padx 3 -pady 3
    pack propagate $fr.__f 0
    focus $fr.__f.b
    place $fr.__f -in $fr -relx 0 -rely 0 -anchor nw -bordermode outside
    
    .central.s process SelectEntities Nodes
    trace var GidPriv(selection) w "EndAddReactions $fr.__f $text ;#"
}

proc EndAddReactions { frame text {iscancel 0 } } {
    global GidPriv ProblemTypePriv
	
    destroy $frame
    foreach i [trace vinfo GidPriv(selection)] {
	trace vdelete GidPriv(selection) [lindex $i 0] [lindex $i 1] 
    }

    if { $iscancel } {
	.central.s process escape escape escape escape
    }

    set curr [.central.s info postprocess get cur_analisis]
    $text del 1.0 end
    if { $ProblemTypePriv(reacormreac) == "reactions" } {
	$text ins end "\n          Reactions list ($curr)\n\n"
    } else { 
	$text ins end "\n       Momentum reactions list ($curr)\n\n"
    }
    $text ins end [format "%6s %10s %10s %10s %10s\n\n"  Node x-comp y-comp z-comp |reac|]

    for { set l 0 } { $l < 3 } { incr  l } { set sum($l) 0.0 }
    set NumNodes 0
    set lastnod ""
    foreach i [lrange [lindex $GidPriv(selection) 0] 1 end] {
	if { ![regexp {([^:]+):(.*)} $i {} ini end] } {
	    set ini $i
	    set end $i
	}
	for { set j $ini } { $j <= $end } { incr j } {
	    if { $j == $lastnod } { continue }
	    set val [.central.s info list_entities Nodes $j]
	    set pos [lsearch $val More]
	    incr pos
	    incr pos
	    set allres [lindex $val $pos]
	    foreach k $allres {
		set resname [lindex $k 0]
		regexp {^(.*)[ ]+[(][^\)]*[)]$} $resname {} resname
		switch $resname {
		    Reactions {
			set reactions [lindex [lindex $k 2] 0]
		    }
		    "M Reactions" {
			set mreactions [lindex [lindex $k 2] 0]
		    }
		}
	    }
	    if { $ProblemTypePriv(reacormreac) == "reactions" } {
		if { [lindex $reactions 0] != "n/a" && [lindex $reactions 0] > -1e35 } {
		    $text ins end [format "%6i %10g %10g %10g %10g\n" $j [lindex $reactions 0] \
			    [lindex $reactions 1] [lindex $reactions 2] [lindex $reactions 3]]
		    for { set l 0 } { $l < 3 } { incr l } {
			set sum($l) [expr $sum($l)+[lindex $reactions $l]]
		    }
		}
	    } else {
		if { [lindex $mreactions 0] != "n/a" && [lindex $mreactions 0] > -1e35 } {
		    $text ins end [format "%6i %10g %10g %10g %10g\n" $j [lindex $mreactions 0] \
			    [lindex $mreactions 1] [lindex $mreactions 2] [lindex $mreactions 3]]
		    for { set l 0 } { $l < 3 } { incr l } {
			set sum($l) [expr $sum($l)+[lindex $mreactions $l]]
		    }
		}
	    }
	    incr NumNodes
	    set lastnod $j
	}
    }
    for { set l 0 } { $l < 54 } { incr l } { $text ins end - }
    $text ins end \n
    set mod [expr sqrt($sum(0)*$sum(0)+$sum(1)*$sum(1)+$sum(2)*$sum(2))]
    $text ins end [format "TOTAL: %10g %10g %10g %10g\n\n" $sum(0) $sum(1) $sum(2) $mod]
    $text ins end "Number of nodes selected: $NumNodes\n"
    $text see end
}

proc AddReactionsWindow { {w .gid.reactions} } {
    global ProblemTypePriv ProgramName


    if { [info procs InitWindow] != "" } {
	InitWindow $w "Reactions calculator" PostReactionsWindowGeom AddReactionsWindowWindow
    } else {
	toplevel $w
    }

    frame $w.f -bd 2 -relief groove

    text $w.f.t -xscroll "$w.f.hscr set" -yscroll "$w.f.vscr set" \
	    -width 50  -font FixedFont

    scrollbar $w.f.vscr -orient vertical -comm "$w.f.t yview"
    scrollbar $w.f.hscr -orient horizontal -comm "$w.f.t xview"

    frame $w.f.mewc
    radiobutton $w.f.mewc.r1 -text "Reactions" -val reactions -var ProblemTypePriv(reacormreac)
    if { $ProgramName == "Rambshell" } {
	radiobutton $w.f.mewc.r2 -text "M Reactions" -val mreactions -var ProblemTypePriv(reacormreac)
	pack $w.f.mewc.r1 $w.f.mewc.r2 -side left
    } else {
	pack $w.f.mewc.r1 -side left
    }
    set ProblemTypePriv(reacormreac) reactions

    grid $w.f.t -row 1 -column 1  -padx 2 -sticky nsew
    grid $w.f.vscr -row 1 -column 2 -sticky ns
    grid $w.f.hscr -row 2 -column 1 -sticky ew -padx 2
    grid $w.f.mewc -row 3 -column 1 -sticky ew -padx 2
    grid columnconf $w.f 1 -weight 1
    grid rowconf $w.f 1 -weight 1

    frame $w.dbuts
    button $w.dbuts.sel -text Select -width 5 -command "AddReactions $w.dbuts $w.f.t" -und 0
    button $w.dbuts.close -text Close -width 5 -command "destroy $w" -und 0

    bind $w.dbuts.sel <Return> "tkButtonInvoke $w.dbuts.sel"
    bind $w <Escape> "tkButtonInvoke $w.dbuts.close"
    bind $w.f.t <Alt-s> "tkButtonInvoke $w.dbuts.sel"
    bind $w.f.t <Alt-c> "tkButtonInvoke $w.dbuts.close"
    bind $w <Alt-s> "tkButtonInvoke $w.dbuts.sel"
    bind $w <Alt-c> "tkButtonInvoke $w.dbuts.close"


    grid $w.f -row 1 -column 1  -pady 4 -sticky ewns \
	    -padx 2
    grid $w.dbuts -row 2 -column 1 -sticky ew
    grid $w.dbuts.sel -in $w.dbuts -row 1 -column 1 -pady 3 -padx 2
    grid $w.dbuts.close -in $w.dbuts -row 1 -column 2 -pady 2 -padx 2


    grid columnconf $w 1 -weight 1
    grid rowconf $w 1 -weight 1

}

proc EndIntegrateSelect { w frame iscancel name1 name2 op } {
    global WarnLineVar GidPriv ProblemTypePriv
    upvar #0 $name1 cad
    
    if { $cad == "Leaving" } {
	destroy $frame
	trace vdelete WarnLineVar w "EndIntegrateSelect $w $frame 0"
	return
    }

    if { $iscancel } {
	destroy $frame
	.central.s process escape
    } elseif {[regexp {from element[ ]+([0-9]+)} $cad {} elem] } {
	#nothing
    } elseif { ![regexp \
	    {Point number[ ]*([^.]*)[^=]*=([^, ]*)[^=]*=([^, ]*)[^=]*=([^, ]*)} \
	    $cad trash pntnum xval yval zval]} { 
	return 
    } else {
	if { $ProblemTypePriv(firstsectionpoint) == "" } {
	    set ProblemTypePriv(firstsectionpoint) $xval,$yval,$zval
	    set ProblemTypePriv(firstsectionpointL) "P1: $xval,$yval,$zval"
	} else {
	    destroy $frame
	    trace vdelete WarnLineVar w "EndIntegrateSelect $w $frame 0"
	    .central.s process escape
	    set ProblemTypePriv(secondsectionpoint) $xval,$yval,$zval
	    set ProblemTypePriv(secondsectionpointL) "P2: $xval,$yval,$zval"
	    set ProblemTypePriv(sectionelems) $GidPriv(selection)
	    IntegrateProcessSelection $w
	}
    }
}

proc IntegrateSelect { w } {
    global WarnLineVar ProblemTypePriv

    set ProblemTypePriv(firstsectionpoint) ""
	
    frame $w.__f -bd 2 -relief ridge -width [expr [winfo width $w]-0] \
	    -height [expr [winfo height $w]-0]
    label $w.__f.l -text "Pick one section (defined by two points) in the shell elements"
    button $w.__f.b -text Cancel -command "EndIntegrateSelect $w $w.__f 1 \
	    WarnLineVar {} w"
    pack $w.__f.l $w.__f.b -side left -padx 3 -pady 3
    pack propagate $w.__f 0
    focus $w.__f.b
    place $w.__f -in $w -relx 0 -rely 0 -anchor nw -bordermode outside
    
    gidmakecurrent
    .central.s process escape escape escape escape utilities id DrawDynLine PointInElement
    trace var WarnLineVar w "EndIntegrateSelect $w $w.__f 0"

}

proc IntegrateObtainResultsFromElements { elems } {
    global Results

    catch { unset Results }

    set validres { {"B Axial Force" N} {BDisplacements m} {Torsor Nm} {"Y' Momentus" Nm} \
	    {"Y' Shear" N} {"Z' Momentus" Nm} {"Z' Shear" N} {"Local Axes Beam" ""} \
	    {"Axial Force" N/m} {Momentus N } {Shear N/m} {"Local Axes" ""} }
    
    foreach elem $elems {
	set val [.central.s info list_entities Elements $elem]
	set Results($elem,nodes) [lindex $val 3]
	set pos [lsearch $val More]
	incr pos
	incr pos
	set allres [lindex $val $pos]
	foreach i $allres {
	    set resname [lindex $i 0]
	    regexp {^(.*)[ ]+[(][^\)]*[)]$} $resname {} resname
	    set found 0
	    set fac ""
	    foreach r $validres {
		if { $resname == [lindex $r 0] } {
		    set found 1
		    break
		}
	    }
	    if { !$found } { continue }
	    set type [lindex $i 1]
	    set Results($elem,$resname) ""
	    switch $type {
		Beams {
		    foreach j [lindex $i 3] {
			foreach l [lindex $j 3] {
			    lappend Results($elem,$resname) $l
			}
		    }				     
		}				     
		Beams1 {			     
		    foreach j [lindex $i 3] {	     
			lappend Results($elem,$resname) [lindex $j 0]
			lappend Results($elem,$resname) [lindex $j 1]
			lappend Results($elem,$resname) [lindex $j 2]
		    }
		}
		Triangles {
		    set inum 1
		    foreach j [lindex $i 3] {
			append Results($elem,$resname) " GP$inum"
			foreach l $j {
			    lappend Results($elem,$resname) $l
			}
			incr inum
		    }
		}
		Triangles1 {
		    foreach j [lindex $i 3] {
			lappend Results($elem,$resname) [lindex $j 0]
			lappend Results($elem,$resname) [lindex $j 1]
			lappend Results($elem,$resname) [lindex $j 2]
		    }
		}
	    }
	}
    }
}


proc IntegrateProcessSelection { w } {
    global ProblemTypePriv Results ResultsInt

    set dir [ConcretePrefs::GiveProjectDir]
    if { $dir == "" } { 
	set dir [ConcretePrefs::GiveTempDir]
	if { $dir == "" } { 
	    WarnWin "Could not find temporal directory. Save project to continue"
	    return
	}
    }
    if { [catch {
	set fin [open [file join $dir tmp_concrete.dat] w]
    }]} {
	WarnWin "error: filed to write to file '[file join $dir tmp_concrete.dat]'"
	return
    }
    puts $fin "problem type Integration"
    puts $fin "PointInElement1 $ProblemTypePriv(firstsectionpoint)"
    puts $fin "PointInElement2 $ProblemTypePriv(secondsectionpoint)"
    
    IntegrateObtainResultsFromElements $ProblemTypePriv(sectionelems)
    
    set nodes ""
    foreach j [array names Results *,nodes] {
	foreach i $Results($j) {
	    if { [lsearch $nodes $i] == -1 } {
		lappend nodes $i
	    }
	}
    }
    set nodes [lsort $nodes]
    
    foreach i $nodes {
	set coords [lindex [.central.s info Coordinates $i] 0]
	set var "Node $i"
	foreach j $coords {
	    lappend var $j
	}
	puts $fin $var
    }

    foreach i $ProblemTypePriv(sectionelems) {
	#it is a beam
	if { ![info exists "Results($i,Local Axes)"] } { continue } 
	puts $fin "element $i $Results($i,nodes)"
	puts $fin "Local Axes $Results($i,Local Axes)"
	foreach j [array names Results $i,*] {
	    if { $j == "$i,Local Axes" } { continue }
	    if { $j == "$i,nodes" } { continue }
	    regexp {,(.*)} $j {} resname
	    puts $fin "$resname $Results($j)"
	}
	puts $fin "end element"
    }
    close $fin


    set exec [file join $ProblemTypePriv(problemtypedir) rambshell]
    WaitMessage $w "Calculating. Wait please..."
    if { [catch {
	catch { file delete [file join $dir tmp_concrete.res] }
	exec $exec -c [file join $dir tmp_concrete.dat]
    }]} {
	EndWaitMessage $w
	set mess "There have been problems in the execution"
	catch {
	    set resfile [file join $dir tmp_concrete.res]
	    set fin [open $resfile r]
	    while { ![eof $fin] } {
		gets $fin aa
		if { [lindex $aa 0] == "Error" } {
		    set mess [lrange $aa 1 end]
		    break
		}
	    }
	    close $fin
	} err
	WarnWin $mess
	return
    }

    EndWaitMessage $w
    set resfile [file join $dir tmp_concrete.res]

    catch { unset ResultsInt }

    set fin [open $resfile r]
    while { ![eof $fin] } {
	gets $fin aa
	if { [string match PointInElem1* $aa] } {
	    set ProblemTypePriv(firstsectionpoint) [lrange $aa 1 end]
	    set ProblemTypePriv(firstsectionpointL) "P1: [lrange $aa 1 end]"
	} elseif { [string match PointInElem2* $aa] } {
	    set ProblemTypePriv(secondsectionpoint) [lrange $aa 1 end]
	    set ProblemTypePriv(secondsectionpointL) "P2: [lrange $aa 1 end]"
	} else {
	    set ResultsInt([lindex $aa 0]) [lrange $aa 1 end]
	}
    }
    close $fin

    set ipos [lsearch [.central.s info gendata] Results_units*]
    set units [lindex [.central.s info gendata] [expr $ipos+1]]
    set m [GiveUnitName $units m]
    set ProblemTypePriv(L) "L: $ResultsInt(L) $m"
    IntUpdateInfo $w ""
}

proc IntFillIntegrals { w } {
    global ResultsInt

    set ipos [lsearch [.central.s info gendata] Results_units*]
    set units [lindex [.central.s info gendata] [expr $ipos+1]]
    set forceu [GiveUnitName $units N]
    set momu [GiveUnitName $units Nm]

    set fam [font configure NormalFont -family]
    set tam [font configure NormalFont -size]
    $w.f.t tag configure subindex -offset -4 -font [list $fam [expr $tam -2]]
    $w.f.t del 1.0 end

    $w.f.t ins end N
    $w.f.t ins end n subindex
    $w.f.t ins end =
    $w.f.t image create end -image [imagefromram intab.gif]
    $w.f.t ins end A
    $w.f.t ins end n subindex
    $w.f.t ins end "dt=$ResultsInt(Int Axial force Nn') $forceu\tn direction\n"

    $w.f.t ins end M
    $w.f.t ins end z subindex
    $w.f.t ins end =
     $w.f.t image create end -image [imagefromram intab.gif]
    $w.f.t ins end A
    $w.f.t ins end n subindex
    $w.f.t ins end "tdt=$ResultsInt(Intz Axial force Nn') $momu\taround z axe\n"

    $w.f.t ins end T
    $w.f.t ins end tn subindex
    $w.f.t ins end =
    $w.f.t image create end -image [imagefromram intab.gif]
    $w.f.t ins end A
    $w.f.t ins end nt subindex
    $w.f.t ins end "dt=$ResultsInt(Int Axial force Ntn') $forceu\tplane tn\n"

    $w.f.t ins end M
    $w.f.t ins end n subindex
    $w.f.t ins end =
    $w.f.t image create end -image [imagefromram intab.gif]
    $w.f.t ins end M
    $w.f.t ins end n subindex
    $w.f.t ins end "dt=$ResultsInt(Int Momentus Mn') $momu\taround t axe\n"

    $w.f.t ins end T
    $w.f.t ins end nz subindex
    $w.f.t ins end =
    $w.f.t image create end -image [imagefromram intab.gif]
    $w.f.t ins end Q
    $w.f.t ins end n subindex
    $w.f.t ins end "dt=$ResultsInt(Int Shear Qn') $forceu\tplane nz\n"

    $w.f.t ins end M
    $w.f.t ins end t subindex
    $w.f.t ins end =
    $w.f.t image create end -image [imagefromram intab.gif]
    $w.f.t ins end M
    $w.f.t ins end tn subindex
    $w.f.t ins end "dt=$ResultsInt(Int Momentus Mtn') $momu\ttorsor around n axe\n"

    $w.f.t ins end "\nNOTES:\n"
    $w.f.t ins end "    Axe t has the direction AB\n"
    $w.f.t ins end "    Axe n is orthogonal to axe t and belong to the plane of the shell\n"
    $w.f.t ins end "    Axe z is equivalent to axe z' of the shell (orthogonal to the shell)\n"
}

proc IntDrawGraphR { w } {
    global ResultsInt ProblemTypePriv

    if { [catch {
	set name $ProblemTypePriv(currentgraph)
	if { $name == "" } { return }
	set shortname [lindex $name end]
	set ipos [lsearch [.central.s info gendata] Results_units*]
	set units [lindex [.central.s info gendata] [expr $ipos+1]]
	set forceum [GiveUnitName $units N/m]
	set momum [GiveUnitName $units Nm/m]
	if { [string match Momentus* $name] } {
	    set unt $momum
	} else { set unt $forceum }
	DrawGraph::DrawCurve $w.c $ResultsInt($name) $ResultsInt(L) L "$shortname ($unt)" $name
    } err] } {
	#WarnWin $err
    }
}

proc IntUpdateInfo { w name } {
    global ResultsInt ProblemTypePriv

    if { $name == "" } {
	if { $ProblemTypePriv(currentgraph) != "" } {
	    set name $ProblemTypePriv(currentgraph)
	} else { set name "Integrals" }
    }

    if { [catch {
	if { $name == "Integrals" } {
	    pack forget $w.c
	    pack $w.f -fill both -expand 1
	    IntFillIntegrals $w
	} else {
	    set shortname [lindex $name end]
	    set ipos [lsearch [.central.s info gendata] Results_units*]
	    set units [lindex [.central.s info gendata] [expr $ipos+1]]
	    set forceum [GiveUnitName $units N/m]
	    set momum [GiveUnitName $units Nm/m]
	    if { [string match Momentus* $name] } {
		set unt $momum
	    } else { set unt $forceum }
	    DrawGraph::DrawCurve $w.c $ResultsInt($name) $ResultsInt(L) L "$shortname ($unt)" $name
	    pack forget $w.f  
	    pack $w.c  -fill both -expand 1
	}
    } err] } {
	WarnWin $err
    }
    set ProblemTypePriv(currentgraph) $name

    $w.mm1.m1 del 2 end
    if { $name == "Integrals" } {
	$w.mm1.m1 add checkbutton -label "Integrals" -underline 0 -command \
		[list IntUpdateInfo $w Integrals] -variable ProblemTypePriv(menuactivebar)
	set ProblemTypePriv(menuactivebar) 1
    } else {
	$w.mm1.m1 add command -label "Integrals" -underline 0 -command \
		[list IntUpdateInfo $w Integrals]
    }
    $w.mm1.m1 add separator

    set str { {"Axial force" Nt' Nn' Ntn'} \
	    {"Momentus" Mt' Mn' Mtn' } \
	    {"Shear" Qt' Qn'}}
    set icount 1
    foreach i $str {
	$w.mm1.m1 add cascade -label [lindex $i 0] -underline 0 -menu $w.mm1.m1.m$icount
	$w.mm1.m1.m$icount del 0 end
	foreach j [lrange $i 1 end] {
	    set iname [concat [lindex $i 0] $j]
	    if { $iname != $name } {
		$w.mm1.m1.m$icount add command -label $j -command \
			[list IntUpdateInfo $w $iname]
	    } else {
		$w.mm1.m1.m$icount add checkbutton -label $j -command \
			[list IntUpdateInfo $w $iname] -variable ProblemTypePriv(menuactivebar)
		set ProblemTypePriv(menuactivebar) 1
	    }
	}
	incr icount
    }

    $w.mm1.m1 add separator
    $w.mm1.m1 add command -label "Close" -underline 0 -command "destroy $w"

    $w.mm1.m2 del 0 end
    $w.mm1.m2 add command -label "Copy" -underline 0 -command "IntCopyCurrentToClipboard $w"
}

proc IntCopyCurrentToClipboard { w } {
    global ResultsInt ProblemTypePriv

    set name $ProblemTypePriv(currentgraph)
    if { $name != "Integrals" } {
	set var "#  x\t$name\n"
	set x 0.0
	foreach i $ResultsInt($name) {
	    append var "$x\t$i\n"
	    set x [expr $x+$ResultsInt(L)/20.0]
	}
	clipboard clear -displayof $w
	clipboard append -displayof $w $var
    } else {
	tk_textCopy $w.f.t
    }
}

proc imagefromram { name } {
    global ProblemTypePriv
    if { ![info exist ProblemTypePriv(image-$name)] } {
	set ProblemTypePriv(image-$name) [image create photo -file \
		[file join $ProblemTypePriv(problemtypedir) images $name]]
    }
    return $ProblemTypePriv(image-$name)
}

proc IntViewSection { what } {
    global ProblemTypePriv

    if { $what == 1 } {
	.central.s process escape escape escape escape utilities signal point FNoJoin \
	    $ProblemTypePriv(firstsectionpoint)
    } else {
	.central.s process escape escape escape escape utilities signal point FNoJoin \
		$ProblemTypePriv(secondsectionpoint)
    }
}

proc CheckScrollbarGrid { text scroll } {

    set bb1 [$text bbox "1.0"]
    set bb2 [$text bbox "end-1c"]

    if { $bb1 != "" && $bb2 != "" } {
	grid remove $scroll
    } else {
	grid $scroll
    }
}

proc IntegrateWindow { {w .gid.integrate} } {
    global ProblemTypePriv ProgramName

    set ProblemTypePriv(currentgraph) ""
    if { [info procs InitWindow] != "" } {
	InitWindow $w "Graphs & Integrate strengths" PostIntegrateWindowGeom IntegrateWindow
    } else {
	toplevel $w
    }

    menu $w.mm1 -type menubar 
    $w conf -menu $w.mm1
    $w.mm1 add cascade -label "View" -menu $w.mm1.m1 -underline 0
    menu $w.mm1.m1
    $w.mm1.m1 add command -label "Select section" -underline 0 -command "IntegrateSelect $w"
    $w.mm1.m1 add separator
    $w.mm1.m1 add command -label "Integrals" -underline 0 -state disabled
    $w.mm1.m1 add separator
    set str { {"Axial force" Nt' Nn' Ntn'} \
	    {"Momentus" Mt' Mn' Mtn' } \
	    {"Shear" Qt' Qn'}}
    set icount 1
    foreach i $str {
	$w.mm1.m1 add cascade -label [lindex $i 0] -underline 0 -menu $w.mm1.m1.m$icount
	menu $w.mm1.m1.m$icount
	foreach j [lrange $i 1 end] {
	    $w.mm1.m1.m$icount add command -label $j -state disabled
	}
	incr icount
    }
    
    $w.mm1.m1 add separator
    $w.mm1.m1 add command -label "Close" -underline 0 -command "destroy $w"

    $w.mm1 add cascade -label "Edit" -menu $w.mm1.m2 -underline 0
    menu $w.mm1.m2
    $w.mm1.m2 add command -label "Copy" -underline 0 -state disabled

    pack [canvas $w.c -relief sunken -bd 2 -bg white -width 400 -height 300] -fill both -expand 1

    frame $w.f 
    label $w.f.l -image [imagefromram intstrengths.gif] -relief groove -bd 2
    frame $w.f.f
    label $w.f.f.l1 -textvar ProblemTypePriv(firstsectionpointL)
    label $w.f.f.l2 -textvar ProblemTypePriv(secondsectionpointL)
    label $w.f.f.l3 -textvar ProblemTypePriv(L)
    button $w.f.f.b1 -text View -width 5 -command "IntViewSection 1"
    button $w.f.f.b2 -text View -width 5 -command "IntViewSection 2"

    grid $w.f.f.l1 -row 1 -column 1 -sticky w -padx 3 -pady 3
    grid $w.f.f.b1 -row 1 -column 2 -sticky w -padx 3 -pady 3
    grid $w.f.f.l2 -row 2 -column 1 -sticky w -padx 3 -pady 3
    grid $w.f.f.b2 -row 2 -column 2 -sticky w -padx 3 -pady 3
    grid $w.f.f.l3 -row 3 -column 1 -sticky w

    set wid [expr [font measure NormalFont "0"]*40]
    text $w.f.t -relief sunken -bd 2 -yscroll "$w.f.sc set" \
	    -tabs "$wid"
    scrollbar $w.f.sc -orient vertical -comm "$w.f.t yview"
    bind $w.f.t <Configure> "CheckScrollbarGrid $w.f.t $w.f.sc"
    bind $w.f.t <KeyPress> "break"

    grid $w.f.f -row 1 -column 1 -padx 3 -pady 3 -sticky w
    grid $w.f.l -row 1 -column 2 -padx 3 -pady 3 -columnspan 2
    grid $w.f.t -row 2 -column 1 -padx 3 -pady 3 -columnspan 2 -sticky ewns
    grid $w.f.sc  -row 2 -column 3 -padx 0 -pady 3 -sticky ns
    grid rowconf $w.f 2 -weight 1
    grid columnconf $w.f 1 -weight 1

    pack $w.f  -fill both -expand 1
    pack forget $w.f

    bind $w.c <Configure> "IntDrawGraphR $w"
    
}

#testing
#  bind all <Control-x><Control-z> {
#      source {c:/Users/ramsan/barres/rambshell.gid/rambshell-2.tcl}
#      if { [catch {
#  	IntegrateWindow
#      } err] } {
#  	WarnWin $err
#      }
#  }


namespace eval DrawGraph {
    variable c
    variable yvalues
    variable maxx
    variable xlabel
    variable ylabel
    variable title
    variable xfact "" xm "" yfact "" ym "" ymax "" ynummin "" ynummax ""

    proc DrawCurve { cv yvaluesv maxxv xlabelv ylabelv titlev } {
	Init $cv $yvaluesv $maxxv $xlabelv $ylabelv $titlev
	Draw
    }
}

proc DrawGraph::Init { cv yvaluesv maxxv xlabelv ylabelv titlev } {
    variable c $cv
    variable yvalues $yvaluesv
    variable maxx $maxxv
    variable xlabel $xlabelv
    variable ylabel $ylabelv
    variable title $titlev
}


proc DrawGraph::Draw {} {
    variable c
    variable yvalues
    variable maxx
    variable xlabel
    variable ylabel
    variable title
    variable xfact
    variable xm
    variable yfact
    variable ym
    variable ymax
    variable ynummin
    variable ynummax

    $c delete curve
    $c delete axestext
    $c delete titletext
    $c delete zeroline

    set inix 0.0
    set numdivisions [expr [llength $yvalues]-1]

    set ymax [winfo height $c]
    set xmax [winfo width $c]

    set ynummax 0
    set textwidth 0
    for {set i 0 } { $i <= $numdivisions } { incr i } {
	set yval [lindex $yvalues $i]
	if { $yval > $ynummax } {
	    set ynummax $yval
	}
	if { $i == 0 || $yval < $ynummin } {
	    set ynummin $yval
	}
    }
    if { $ynummax == $ynummin } { incr ynummax }

    set inumtics 8
    for {set i 0 } { $i < $inumtics } { incr i } {
	set yvaltext [format "%.4g" [expr $ynummin+$i/double($inumtics-1)*($ynummax-$ynummin)]]
	regsub {e([+-])00} $yvaltext {e\10} yvaltext
	set tt [font measure NormalFont $yvaltext]
	if { $tt > $textwidth } { set textwidth $tt }
    }

    set xm [expr $textwidth+10]
    set textheight [font metrics NormalFont -linespace]
    set ym [expr int($textheight*1.5)+10]

    set fam [font configure NormalFont -family]
    set tsize [expr [font configure NormalFont -size]*2]
    $c create text [expr $xmax/2] 6 -anchor n -justify center \
	    -text $title -font [list $fam $tsize] -tags titletext
    
    $c create line $xm $ym $xm [expr $ymax-$ym] [expr $xmax-$xm] \
	    [expr $ymax-$ym]  -tags axestext

    set inumtics 8
    for {set i 0 } { $i < $inumtics } { incr i } {
	set xvaltext [format "%3.1lf" \
		[expr $inix+$i/double($inumtics-1)*($maxx-$inix)]]
	set xval [expr $xm+$i/double($inumtics-1)*($xmax-2*$xm)]
	set xvalt $xval
	if { $i == 0 } { set xvalt [expr $xvalt+4] }
	$c create line $xval [expr $ymax-$ym+2] $xval [expr $ymax-$ym] \
		-tags axestext
	$c create text $xval [expr $ymax-$ym+2] -anchor n -justify center \
		-text $xvaltext -font NormalFont -tags axestext

	set yvaltext [format "%.4g" [expr $ynummin+$i/double($inumtics-1)*($ynummax-$ynummin)]]
	regsub {e([+-])00} $yvaltext {e\10} yvaltext
	set yval [expr $ymax-$ym-$i/double($inumtics-1)*($ymax-2*$ym)]
	set yvalt $yval
	if { $i == 0 } { set yvalt [expr $yvalt-4] }
	$c create line [expr $xm-2] $yval $xm $yval -tags axestext
	$c create text [expr $xm-3] $yvalt -anchor e -justify right \
		-text $yvaltext -font NormalFont -tags axestext
    }
    $c create text 6 6 -anchor nw -justify left \
	    -text $ylabel -font NormalFont -tags axestext
    set textwidth [font measure NormalFont 0.0]
    $c create text [expr $xmax-$xm+$textwidth] [expr $ymax-$ym] \
	    -anchor nw -justify left \
	    -text $xlabel -font NormalFont -tags axestext

    set xfact [expr ($xmax-2.0*$xm)/double($maxx-$inix)]
    set yfact [expr ($ymax-2.0*$ym)/double($ynummax-$ynummin)]

    set yval [expr $ymax-$ym-(0.0-$ynummin)*$yfact]
    if { $yval > $ym && $yval <= [expr $ymax-$ym] } {
	$c create line $xm $yval [expr $xmax-$xm] $yval -tags zeroline -dash -.-
    }

    set xfactdiv [expr ($xmax-2.0*$xm)/double($numdivisions)]
    set lastyval [expr $ymax-$ym-([lindex $yvalues 0]-$ynummin)*$yfact]
    for {set i 1 } { $i <= $numdivisions } { incr i } {
	set yval [expr $ymax-$ym-([lindex $yvalues $i]-$ynummin)*$yfact]
	$c create line [expr ($i-1)*$xfactdiv+$xm] $lastyval \
		[expr $i*$xfactdiv+$xm] $yval -tags curve -width 3 -fill red
	set lastyval $yval
    }

    $c bind curve <ButtonPress-1> "DrawGraph::DrawGraphCoords %x %y"
}

proc DrawGraph::FindClosestPoint { x y } {
    variable c

    set mindist2 1e20
    foreach i [$c find withtag curve] {
	foreach "ax ay bx by" [$c coords $i] break
	set vx [expr $bx-$ax]
	set vy [expr $by-$ay]
	set alpha [expr $vx*($ax-$x)+$vy*($ay-$y)]
	set landa [expr -1*$alpha/double($vx*$vx+$vy*$vy)]
	if { $landa < 0.0 } { set landa 0.0 }
	if { $landa > 1.0 } { set landa 1.0 }
	set px [expr $ax+$landa*$vx]
	set py [expr $ay+$landa*$vy]

	set dist2 [expr ($px-$x)*($px-$x)+($py-$y)*($py-$y)]
	if { $dist2 < $mindist2 } {
	    set mindist2 $dist2
	    set minpx $px
	    set minpy $py
	}
    }
    return [list $minpx $minpy]
}

proc DrawGraph::DrawGraphCoords { x y } {
    variable c
    variable xlabel
    variable ylabel
    variable xfact
    variable xm
    variable yfact
    variable ym
    variable ymax
    variable ynummin
    variable ynummax
    variable yvalues

    $c delete coords
    $c delete coordpoint

    set ymax [winfo height $c]
    set xmax [winfo width $c]
    
    if { [lindex $yvalues end] < ($ynummax-$ynummin)/2.0 } {
	foreach "{} {} {} ytitle" [$c bbox titletext] break
	set ytitle [expr $ytitle+2]
	set anchor ne
    } else {
	set ytitle [expr $ymax-$ym-5]
	set anchor se
    }

    foreach "xcurve ycurve" [DrawGraph::FindClosestPoint $x $y] break

    $c create oval [expr $xcurve-2] [expr $ycurve-2] [expr $xcurve+2] [expr $ycurve+2] \
	    -tags coordpoint

    set xtext [expr ($xcurve-$xm)/double($xfact)]
    regsub {e([+-])00} $xtext {e\10} xtext
    set ytext [expr ($ymax-$ym-$ycurve)/double($yfact)+$ynummin]
    regsub {e([+-])00} $ytext {e\10} ytext

    $c create text [expr $xmax-6] $ytitle -anchor $anchor -font NormalFont \
	    -text [format "$xlabel: %.4g  $ylabel: %.4g" $xtext $ytext] -tags coords

    $c bind curve <ButtonRelease-1> "$c delete coords coordpoint; $c bind curve <B1-Motion> {}"
    $c bind curve <B1-Motion> "DrawGraph::DrawGraphCoords %x %y"
}

namespace eval DisconnectConditions {
    variable Identifier
}

proc DisconnectConditions::ComunicateWithGiD { op args } {
    variable Identifier
    switch $op {
	"INIT" {
	    set PARENT [lindex $args 0]
	    upvar      [lindex $args 1] ROW 
	    set GDN [lindex $args 2]
  	    set STRUCT [lindex $args 3]

	    global $GDN
	    upvar \#0 $GDN GidData

	    set cndname [lindex [split $STRUCT ,] 1]
	    switch $cndname {
		Point_disconnect_degrees - Line_disconnect_degrees {
		    set Identifier $GidData($STRUCT,VALUE,2)
		}
		Line_disconnect_Id - Surface_disconnect_Id {
		    set Identifier $GidData($STRUCT,VALUE,1)
		}
		default {
		    if { ![info exists Identifier] } {
			set Identifier -ANY-
		    }
		}
	    }

	    set l [label $PARENT.l$ROW -text "Identifier:"]
	    set c [ComboBox $PARENT.c$ROW \
		    -textvariable DisconnectConditions::Identifier \
		    -values {-ANY- Id1 Id2 Id3 Id4 Id5 Id6 Id7 Id8}]
	    
	    grid $l -row $ROW -column 0 -sticky e
	    grid $c -row $ROW -column 1 -sticky w -pady 3

	    if { [info exists $GDN\($STRUCT,HELP,0\)] } {
		GidHelp "$l $c" [subst $GidData($STRUCT,HELP,0)]
	    }
	    return ""
	}
	"SYNC" {
	    set GDN [lindex $args 0]
  	    set STRUCT [lindex $args 1]

	    set Identifier [string trim $Identifier]
	    regsub -all " \t"  $Identifier {} Identifier
	    if { $Identifier != "-ANY-" && ![regexp {^[a-zA-Z0-9]} $Identifier] } {
		return [list ERROR \
			"Identifier must be -ANY- or a name that begins with a letter or number"]
	    }
	    DWLocalSetValue $GDN $STRUCT Identifier $Identifier
	    return ""
	}
    }
}